@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/pd/uuwho:RELEASE-3_2_0_106:uuwho.c,v 1.19 1998/11/06 19:56:14 woods Exp"
+#ident	"@(#)smail/pd/uuwho:RELEASE-3_2_0_107:uuwho.c,v 1.21 1999/07/19 01:34:02 woods Exp"
  */
 /*
  * uuwho:
@@ -24,6 +24,7 @@
 
 #include <stdio.h>
 #include <ctype.h>
+
 #include "defs.h"
 
 #ifdef HAVE_STDLIB_H
@@ -88,8 +89,13 @@ static int rawformat = 0;
 
 extern int errno;
 extern long atol();
-extern char *malloc();
-extern char *realloc();
+#if defined(ANSI_C) && !defined(NO_VOID_MALLOC)
+extern void	*malloc();
+extern void	*realloc();
+#else
+extern char	*malloc();
+extern char	*realloc();
+#endif
 
 int
 main(argc, argv)
@@ -564,7 +570,6 @@ getfile(filename)
     register char *cp;		/* pointer */
     register int namelen;	/* length of sitename */
     static int input=0;		/* 1 ==> already read input */
-    char *malloc();		/* storage allocator */
     int linelen;		/* the length of the current line */
     long pos = 0;		/* current file position */
 #ifdef UUWHO_USE_DBM
@@ -790,7 +795,6 @@ static char *
 get_hostname()
 {
     char *hostname;			/* who we are */
-    char *malloc();			/* memory allocator */
 
     /*
      * My man page says that 255 chars (plus nul byte) is the limit
