@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:addr.c,v 1.27 1998/09/21 05:45:48 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:addr.c,v 1.29 1999/09/07 21:56:26 woods Exp"
  */
 
 /*
@@ -22,6 +22,9 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <ctype.h>
+
+#define NEED_SOCKETS			/* Required for IP address validation */
+
 #include "defs.h"
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
@@ -36,6 +39,19 @@
 # include "extern.h"
 #endif
 
+#ifndef INADDR_NONE
+# define INADDR_NONE		((unsigned long) (-1)) /* XXX 64bit too??? */
+#endif
+#ifndef INADDR_LOOPBACK
+# define INADDR_LOOPBACK	((unsigned long) 0x7f000001) /* XXX not 64-bit clean */
+#endif
+
+#ifdef HAVE_BSD_NETWORKING
+# ifndef OBSOLETE_RESOLVER
+extern int h_errno;			/* import h_errno; many systems don't define it in <netdb.h> */
+# endif
+#endif
+
 /* functions local to this file */
 static int check_target_and_remainder();
 static char *escaped();
@@ -506,8 +522,8 @@ check_target_and_remainder(target, remainder)
     char **target;
     char **remainder;
 {
-    register int c;
-    register char *p;
+    int c;
+    char *p = *target;
 
     if ((*remainder)[0] == '\0') {
 	*remainder = "no remainder address";
@@ -515,21 +531,44 @@ check_target_and_remainder(target, remainder)
 	return FAIL;
     }
     /* the set of chars in the target should be limited to a small set */
-    p = *target;
     if (*p == '-') {
 	*remainder = "target cannot begin with `-'";
-	DEBUG1(DBG_ADDR_MID, "parse_address: %s\n", *remainder);
+	DEBUG2(DBG_ADDR_MID, "parse_address(%s): %s\n", p, *remainder);
 	return FAIL;
     }
     if (*p == '[') {
+#ifdef HAVE_BSD_NETWORKING
+# if defined(INET_ADDR_USE_STRUCT)
+	struct in_addr s_inet;		/* internet address */
+# endif
+	unsigned long inet;		/* IP address */
+	char *p2;			/* pointer to closing bracket (]) */
+
+	p2 = strchr(p, ']');
+	*p2 = '\0';
+# ifdef INET_ADDR_USE_STRUCT
+	s_inet = inet_addr(&(p[1]));
+	inet = s_inet.s_addr;
+# else
+	inet = inet_addr(&(p[1]));
+# endif
+	*p2 = ']';
+	DEBUG3(DBG_ADDR_HI, "parse_address(%s): inet given: [0x%lx] aka %s\n", p,
+	       ntohl(inet), COPY_STRING(inet_ntoa(*((struct in_addr *) &inet))));
+	if (inet == INADDR_NONE) {
+	    *remainder = "Invalid host address form";
+	    DEBUG2(DBG_ADDR_LO, "parse_address(%s): inet_addr() failed: %s.\n", p, *remainder);
+	    return FAIL;
+	}
+#else /* !HAVE_BSD_NETWORKING */
+	/* XXX shouldn't be lazy -- should verify the hard way! */
+#endif
 	return SUCCEED;
     }
     while ((c = *p++)) {
-	if ( !(isalnum(c) || c == '.' || c == '-' || c == '_' ||
-	       c == '+' || c == '=') )
-	{
+	if (!(isalnum(c) || c == '.' || c == '-' || c == '_' || c == '+' || c == '=')) {
 	    *remainder = "illegal character in hostname";
-	    DEBUG1(DBG_ADDR_MID, "parse_address: %s\n", *remainder);
+	    DEBUG2(DBG_ADDR_MID, "parse_address(%s): %s\n", *target, *remainder);
 	    return FAIL;
 	}
     }
