@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:bindlib.c,v 1.58 1999/03/25 22:24:39 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:bindlib.c,v 1.60 1999/07/22 05:07:36 woods Exp"
  */
 
 /*
@@ -525,15 +525,16 @@ bind_check_if_canonical_host(hostnm, inet, error_p)
 	    if ((a_rr->rr_type == T_A) && (memcmp(a_rr->rr_data, (char *) &inet, sizeof(inet)) == 0)) {
 		result = DB_SUCCEED;	/* Oh!  I guess there was a matching A RR! */
 	    } else if (a_rr->rr_type == T_CNAME) {
-		*error_p = not_a_canonical_host("that domain name is a CNAME not a hostname");
+		*error_p = not_a_canonical_host(xprintf("the domain name '%s' is a CNAME not a hostname (it must be a domain name resolving to an A record)", hostnm));
 	    } /* we don't care about any other kind of RRs */
 	}
     }
     if (result != DB_SUCCEED && !*error_p) {
-	*error_p = not_a_canonical_host(xprintf("no A records for that hostname match the target address '%s'",
+	*error_p = not_a_canonical_host(xprintf("no A records for the hostname '%s' have a target address matching the source address [%s]",
+						hostnm,
 						inet_ntoa(*((struct in_addr *) &inet))));
     } else if (nothing) {
-	*error_p = not_a_canonical_host("DNS query did not return any records with a matching hostname");
+	*error_p = not_a_canonical_host(xprintf("A DNS query did not return any records matching the hostname '%s'", hostnm));
     }
     xfree((char *) a_rrs);
     _res.options = save_res_options;
@@ -630,6 +631,13 @@ bind_check_if_local_mxs(target, local_precedence_p, error_p)
 
         return ret;
     }
+    if ((ret = get_addr_hints(mxdat)) != DB_SUCCEED) {
+	DEBUG1(DBG_DRIVER_LO, "problem filling in address hints for %s.\n", strip_target);
+	mx_work_area_destructor(mxdat, FALSE);
+	_res.options = save_res_options;
+
+        return ret;
+    }
     ret = DB_SUCCEED;
     *local_precedence_p = mxdat->local_precedence;
 
@@ -892,7 +900,7 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
                     struct mx_work_area * mxdat2;
         
                     mxdat2 = mx_work_area_constructor(priv, error_p, what, flags, &found_server, &no_server);
-                       result = full_mx_lookup(priv->uk_greybook_relay, mxdat2, FALSE, priv);
+		    result = full_mx_lookup(priv->uk_greybook_relay, mxdat2, FALSE, priv);
                     if (result == DB_SUCCEED) {
                         if (gateway == NULL) {
                             if (priv->rewrite_headers >= (inverting ? BIND_REWRITE_IFREQUIRED : BIND_REWRITE_ALWAYS))
@@ -1240,8 +1248,9 @@ static int decode_cnames(mxdat)
 
     do {                                /* Iterate through any CNAMEs */
         rewindrr(&mx_it, mxdat->mx_rrs, mxdat->mx_size);
-        while ((mx_rr = getnextrr(&mx_it, FALSE)) && mx_rr->rr_type != T_CNAME)
+        while ((mx_rr = getnextrr(&mx_it, FALSE)) && mx_rr->rr_type != T_CNAME) {
             ;
+	}
         if (mx_rr != NULL) {
             static char nambuf[MAXDNAME];
             int dlen;
@@ -1399,15 +1408,15 @@ static int make_mx_hints(mxdat)
 
             add_mx_hint(&(mxdat->mx_hints), precedence, name, FALSE);
 
-            /* Extract the full name of the target if poss */
+            /* Extract the full name of the target if possible */
             if (mxdat->full_target == NULL) {
                 mxdat->full_target = COPY_STRING(mx_it.dname);
             }
 
             /* Pick up the local host's precedence */
-            if (islocalhost(name) && 
-                (mxdat->local_precedence < 0 || precedence < mxdat->local_precedence))
+            if (islocalhost(name) && (mxdat->local_precedence < 0 || precedence < mxdat->local_precedence)) {
                 mxdat->local_precedence = precedence;
+	    }
         }
     }
     return DB_SUCCEED;
@@ -1420,9 +1429,9 @@ static int make_mx_hints(mxdat)
  *                        Basically this means making sure that there
  *                        no MX records with precedence higher than that
  *                        of the local host.
- *                        If you want UK behaviour (poor thing), then tossing
- *                        of records using "banned" gateways or exccessive
- *                        preference values are also tossed.
+ *                        If you want UK behaviour (poor thing) then toss
+ *                        records using "banned" gateways and any with
+ *                        exccessive preference values.
  */
 
 static int filter_mx_hints(mxdat)
@@ -1433,7 +1442,7 @@ static int filter_mx_hints(mxdat)
 
 
 #define mx_hint ((struct mx_transport_hint *)((*mx_a)->private))
-    for(mx_a = &(mxdat->mx_hints); (*mx_a);) {
+    for (mx_a = &(mxdat->mx_hints); (*mx_a);) {
         char *name = mx_hint->exchanger;
         int precedence = mx_hint->preference;
 
