@@ -1,4 +1,4 @@
-/* @(#) child.h,v 1.3 1992/07/11 11:48:03 tron Exp */
+/* @(#) child.h,v 1.4 1998/04/12 01:59:48 woods Exp */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -15,10 +15,11 @@
 
 /* bit values for the flag parameter to open_child */
 #define CHILD_DUPERR	0x0001		/* duplicate stdout to stderr */
-#define CHILD_DEVNULL	0x0002		/* use "/dev/null" */
-#define CHILD_RETRY	0x0004		/* retry the fork() if it failes */
-#define CHILD_MINENV	0x0008		/* supply a default minimum env */
-#define CHILD_NOCLOSE	0x0010		/* don't close extraneous fd's */
+#define CHILD_CLSERR	0x0002		/* close stderr and open on /dev/null */
+#define CHILD_DEVNULL	0x0004		/* use "/dev/null" */
+#define CHILD_RETRY	0x0008		/* retry the fork() if it failes */
+#define CHILD_MINENV	0x0010		/* supply a default minimum env */
+#define CHILD_NOCLOSE	0x0020		/* don't close extraneous fd's */
 
 #ifndef	FORK_RETRIES
 # define FORK_RETRIES	10		/* default, 10 retries */
