@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:config.c,v 1.35 1998/11/02 00:13:43 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:config.c,v 1.37 1999/08/08 16:46:11 woods Exp"
  */
 
 /*
@@ -31,6 +31,7 @@
 #endif
 
 char *listen_name = NULL; 		/* DNS name of interface to listen on */
+ char *sending_name = NULL; 		/* DNS name of interface to send mail from */
 char *visible_name = VISIBLE_NAME;	/* hostname used in outgoing addrs */
 char *visible_domains = VISIBLE_DOMAINS; /* domain used in outgoing headers */
 char *uucp_name = UUCP_NAME;		/* hostname used in !-routes */
@@ -108,6 +109,7 @@ int smtp_hello_verify_literal = SMTP_HELLO_VERIFY_LITERAL; /* verify HELO/EHLO d
 int smtp_hello_verify_ptr = SMTP_HELLO_VERIFY_PTR; /* verify HELO/EHLO matches PTR RR for sockaddr */
 int smtp_sender_verify_mx_only = SMTP_SENDER_VERIFY_MX_ONLY; /* require MX RRs for MAIL FROM: hosts */
 int smtp_max_recipients = SMTP_MAX_RECIPIENTS; /* maximum recipients per SMTP connect */
+int debug_pause_for_smtp_connections = 0; /* seconds to pause after child fork for debugger connect */
 char *sender_env_variable = SENDER_ENV_VARIABLE; /* env variable naming user */
 int switch_percent_and_bang = SWITCH_PERCENT_AND_BANG;
 					/* switch precedence of % and ! */
