@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:direct.c,v 1.26 1998/11/20 01:01:24 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:direct.c,v 1.27 1999/07/22 05:04:30 woods Exp"
  */
 
 /*
@@ -272,14 +272,14 @@ direct_local_addrs(in, out, retry, defer, fail)
 	    } else if (EQIC(in->remainder, "Postmaster")) {
 		implicit_redirect(in, retry, postmaster_address);
 	    } else {
-		/* no director found the user, fail the address */
+		/* no director found the mailbox, fail the address */
 		exitvalue = EX_NOUSER;	/* also set exit status */
 		/*
 		 * ERR_100 - unknown user
 		 *
 		 * DESCRIPTION
-		 *	The remainder structure in this addr structure was
-		 *      not matched by any of the directors.
+		 *	The remainder string in this addr structure was not
+		 *	matched by any of the directors.
 		 *
 		 * ACTIONS
 		 *      Send a message to the owner of the address, or to
@@ -289,9 +289,9 @@ direct_local_addrs(in, out, retry, defer, fail)
 		 *      A list owner should check through the list that he
 		 *      or she owns.  A sender may wish to send mail to the
 		 *      postmaster of the machine asking about login names
-		 *      for a particular user.
+		 *      or aliases for a particular user.
 		 */
-		in->error = note_error(ERR_NSOWNER|ERR_100, "unknown user");
+		in->error = note_error(ERR_NSOWNER|ERR_100, "no such user or mailbox");
 		in->succ = *fail;
 		*fail = in;
 	    }
