@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_106:aliasfile.c,v 1.20 1998/11/20 01:00:01 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_107:aliasfile.c,v 1.22 1999/07/22 05:15:22 woods Exp"
  */
 
 /*
@@ -333,10 +333,10 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 				 (char *)NULL, &list, F_ALIAS, &error);
 	    /*
 	     * If a director owner exists, compute the owner address for this
-	     * mailing list and add it to each generated address.  This is for
-	     * the benefit of outgoing SMTP, which wants to use it in the
-	     * envelope for error returns.  However, don't waste time if there
-	     * aren't any addresses to set the owner for!
+	     * alias and add it to each generated address.  This is for the
+	     * benefit of outgoing SMTP, which wants to use it in the envelope
+	     * for error returns.  However, don't waste time if there aren't
+	     * any addresses to set the owner for!
 	     */
 	    if (!(dp->owner) ||
 		!(owner = expand_string(dp->owner,
@@ -357,7 +357,7 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 		 * for list handling.  So make a copy beforehand.  Don't waste
 		 * time verifying "postmaster", which should always be valid.
 		 */
-		owner = copy(preparsed_owner); /* copy to use later */
+		owner = COPY_STRING(preparsed_owner); /* copy to use later */
          
 		if (strcmpic(preparsed_owner, "postmaster") != 0) { 
 		    struct addr *new2 = alloc_addr();
