@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_106:fwdfile.c,v 1.30 1998/11/20 01:00:02 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_107:fwdfile.c,v 1.31 1999/07/22 05:15:31 woods Exp"
  */
 
 /*
@@ -493,8 +493,8 @@ finish_forward(dp, addr, new, defer, fail)
 	struct addr *fail2;
 	struct addr *okay = NULL;
 
-	fail2 = fail2_safe;			/* verify_addr_list can clobber last arg */
-	owner = copy(preparsed_owner);		/* copy to use later */
+	fail2 = fail2_safe;		/* verify_addr_list can clobber last arg */
+	owner = COPY_STRING(preparsed_owner); /* copy to use later */
 
 	new2->in_addr = preparsed_owner;
 	new2->work_addr = preparsed_owner;
