@@ -1,4 +1,4 @@
-/* @(#) $Id: fwdfile.h,v 1.3 1992/07/11 11:48:35 tron Exp $ */
+/* @(#) fwdfile.h,v 1.4 1995/03/01 13:47:31 nm4 Exp */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -7,7 +7,7 @@
  * See the file COPYING, distributed with smail, for restriction
  * and warranty information.
  *
- * namei master id: @(#) $Id: fwdfile.h,v 1.3 1992/07/11 11:48:35 tron Exp $
+ * namei master id: @(#) fwdfile.h,v 1.4 1995/03/01 13:47:31 nm4 Exp
  */
 
 /*
@@ -18,6 +18,7 @@
 /* boolean attributes for director.flags field */
 #define FWD_CHECKOWNER	0x00010000
 #define FWD_FORWARDTO	0x00020000
+#define FWD_LOCKFWDFILE	0x00040000
 
 /* private information from director file entry */
 struct forwardfile_private {
