@@ -1,4 +1,4 @@
-/* @(#) $Id: smartuser.h,v 1.3 1992/07/11 11:48:44 tron Exp $ */
+/* @(#) smartuser.h,v 1.4 1995/05/30 16:16:29 nm4 Exp */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -19,4 +19,6 @@
 /* private information from director file entry */
 struct smartuser_private {
     char *new_user;			/* template for new address */
+    char *transport;			/* transport if routing to "smart" transport */
+    struct transport *transport_ptr;	/* transport structure */
 };
