@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:dys.h,v 1.8 1996/02/28 14:26:25 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:dys.h,v 1.9 1999/07/19 01:37:56 woods Exp"
  */
 
 /*
@@ -55,6 +55,11 @@ struct str {
 	 ((sp)->i = 0),					\
 	 ((sp)->p = xmalloc((sp)->a)))
 
+/*
+ * STR_CLEAR - prepare to re-use a dynamic string
+ */
+#define STR_CLEAR(sp)	((sp)->i = 0)
+
 /*
  * STR_NEXT - allow access to the next character in a dynamic string
  * region, growing the region if no successor character currently
@@ -73,6 +78,11 @@ struct str {
 	    (sp)->p[(sp)->i++] = (c);			\
 	}
 
+/*
+ * STR_PREV - remove the previous character from a dynamic string
+ */
+#define STR_PREV(sp)	((sp)->i--)
+
 /*
  * STR_CAT - concatenate a string onto the end of a dynamic string
  * region, growing as necessary.
@@ -104,4 +114,4 @@ struct str {
 /*
  * COPY_STRING - copy a C-style string to a new xmalloc'd region
  */
-#define COPY_STRING(s)	(strcpy(xmalloc((unsigned) (strlen((s)) + 1)), (s)))
+#define COPY_STRING(sp)	(strcpy(xmalloc((unsigned) (strlen((sp)) + 1)), (sp)))
