@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:error.h,v 1.16 1998/09/27 00:38:56 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:error.h,v 1.18 1999/08/14 05:32:52 woods Exp"
  */
 
 /*
@@ -14,7 +14,8 @@
  * error.h:
  *	error numbers used by smail, for address structures or for
  *	other purposes.  Error numbers up to 1000 are reserved.
- *	Numbers over 1000 may be used for local purposes.
+ *	Numbers from 1000 to 0xFFFF may be used for local purposes.
+ *	Bits above 0xFFFFL are reserved for flags (see addr.h).
  */
 #define ERR_100		100L		/* unknown user */
 #define ERR_101		101L		/* unknown host */
@@ -103,3 +104,8 @@
 #define ERR_184		184L		/* resolve timeout - too long in directors/routers */
 #define ERR_185		185L		/* altuser passwd file is unparsable */
 #define ERR_186		186L		/* canonical host not found */
+#define ERR_187		187L		/* smtp remote mailbox busy */
+#define ERR_188		188L		/* smtp remote error in processing */
+#define ERR_189		189L		/* smtp user not local */
+#define ERR_190		190L		/* smtp user over quota */
+#define ERR_191		191L		/* smtp illegal user */
