@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:expand.c,v 1.29 1999/02/03 06:06:31 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:expand.c,v 1.30 1999/07/19 01:39:51 woods Exp"
  */
 
 /*
@@ -1019,7 +1019,7 @@ substitute(addr, file, found, var, len)
     register char *var;			/* start of variable */
     register int len;			/* length of variable */
 {
-    static char buf[50];
+    static char buf[50];		/* big enough for a %ld expansion */
 
 #define MATCH(x) (len==sizeof(x)-1 && strncmpic(var, x, sizeof(x)-1) == 0)
 
@@ -1124,10 +1124,8 @@ substitute(addr, file, found, var, len)
 	return get_arpa_date(message_date());
     }
     if (MATCH("$") || MATCH("pid")) {
-	static char pidbuf[10];
-
-	(void) sprintf(pidbuf, "%d", getpid());
-	return pidbuf;
+	(void) sprintf(buf, "%d", getpid());
+	return buf;
     }
     if (MATCH("uucp_name")) {
 	return uucp_name;
