@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:extern.h,v 1.67 1998/11/20 02:21:49 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:extern.h,v 1.69 1999/08/08 16:46:12 woods Exp"
  */
 
 /*
@@ -25,6 +25,7 @@ extern int x_dont_panic;                /* dont panic in X_CHECK() */
 
 /* external variables defined in config.c */
 extern char *listen_name;		/* DNS name of interface to listen on */
+extern char *sending_name;		/* DNS name of interface to send mail from */
 extern char *visible_name;		/* hostname used in outgoing addrs */
 extern char *visible_domains;		/* domain used in outgoing headers */
 extern char *uucp_name;			/* hostname used in !-routes */
@@ -103,6 +104,7 @@ extern int smtp_hello_verify_literal;	/* verify HELO/EHLO domain literal has PTR
 extern int smtp_hello_verify_ptr;	/* verify HELO/EHLO matches PTR RR for sockaddr */
 extern int smtp_sender_verify_mx_only;	/* require MX RRs for MAIL FROM: hosts */
 extern int smtp_max_recipients;		/* maximum recipients per SMTP connect */
+extern int debug_pause_for_smtp_connections; /* seconds to pause after child fork for debugger connect */
 extern char *sender_env_variable;	/* env variable naming user */
 extern int switch_percent_and_bang;	/* switch precedence of % and ! */
 extern int error_copy_postmaster;	/* copy postmaster on errors */
@@ -237,9 +239,6 @@ extern char *bat;			/* the proper bat for this release */
 extern char *path_to_sender;		/* uucp-style route to sender */
 extern int cached_transports;		/* TRUE if cache_transports() called */
 
-/* external variables defined in route.c */
-extern int cached_routers;		/* TRUE if cache_routers() called */
-
 /* external variables defined in silly.c */
 extern void silly();			/* a very silly function */
 
@@ -348,7 +347,6 @@ extern int lookup_database();
 extern int match_ip();
 
 /* external functions defined in main.c */
-extern int main();
 extern void initialize_state();
 extern void process_args();
 
@@ -371,22 +369,7 @@ extern void print_queue();
 extern void smtp_mode();
 extern int process_spool_file();
 extern int fork_wait();
-
-/* external functions defined in notify.c */
-extern void build_host_strings();
-extern void compute_nobody();
-extern void input_signals();
-extern void process_signals();
-extern void delivery_signals();
-extern void test_addresses();
-extern void perform_deliver_mail();
-extern void deliver_mail();
 extern void run_queue();
-extern void print_version();
-extern void print_queue();
-extern void smtp_mode();
-extern void batch_smtp_mode();
-extern void hash_predelivered_addresses();
 
 /* external functions defined in notify.c */
 extern void fail_delivery();
@@ -395,6 +378,7 @@ extern void succeed_delivery();
 extern void error_delivery();
 extern struct addr *process_msg_log();
 extern void notify();
+extern void hash_predelivered_addresses();
 
 /* external functions defined in parse.c */
 extern char *parse_entry();
@@ -413,10 +397,6 @@ extern struct group *getgrbyname();
 extern struct passwd *getpwbyuid();
 extern struct group *getgrbygid();
 
-/* external functions defined in qualify.c */
-extern char *read_qualify_file();
-extern char *qualify_domain();
-
 /* external functions defined in queue.c */
 extern int queue_message();
 extern char **read_message();
@@ -507,6 +487,7 @@ extern char *xprintf();
 extern int dprintf();
 extern long c_atol();
 extern char *base62();
+extern char *read_line();
 extern void str_cat();
 extern void str_ncat();
 #ifndef HAVE_VFPRINTF
