@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:log.c,v 1.23 1998/04/11 23:30:26 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:log.c,v 1.24 1999/07/22 05:03:15 woods Exp"
  */
 
 /*
@@ -680,36 +680,36 @@ send_log(f, all, banner)
     char *banner;			/* if non-NULL precedes log output */
 {
     FILE *mlog;				/* message log file */
-    register int c;			/* for copying data between files */
-    int last_c;
+    char *mln;				/* line from log file */
+    int lastx = FALSE;			/*  */
 
     build_msg_log_fn();
-    mlog = fopen(msg_log_fn, "r");	/* reopen log file */
-    if (mlog == NULL) {
-	return;				/* no message file found */
+    /* reopen log file to avoid changing the write position */
+    if (! (mlog = fopen(msg_log_fn, "r"))) {
+	DEBUG2(DBG_LOG_LO, "send_log(): no message log file: %s: %s.\n", msg_log_fn, strerror(errno));
+	return;
     }
-    last_c = '\n';
-    while ((c = getc(mlog)) != EOF) {	/* copy from log to specified file */
-	if (last_c == '\n' && c == 'X' && !all) {
-	    /* ignore lines beginning with X */
-	    while ((c = getc(mlog)) != EOF && c != '\n') ;
-	    if (c == EOF) {
-		break;
-	    }
-	    last_c = '\n';
+    /* copy from message-log to the specified file */
+    while ((mln = read_line(mlog)) && *mln) {
+	if (*mln == 'X' && !all) {
+	    lastx = TRUE;
 	    continue;
 	}
-	if (last_c == '\n') {
-	    if (banner) {
-		(void)fputs(banner, f);	/* output banner at most once */
-		banner = NULL;
+	if (lastx) {
+	    /* ignore any following SMTP replies too */
+	    if (strlen(mln) >= 3 && isdigit(mln[0]) && isdigit(mln[1]) && isdigit(mln[2])) {
+		continue;
+	    } else {
+		lastx = FALSE;
 	    }
-	    (void)fputs(" ", f);
 	}
-	putc(c, f);
-	last_c = c;
+	if (banner) {
+	    (void) fputs(banner, f);
+	    banner = NULL;		/* output banner at most once */
+	}
+	(void) fprintf(f, " %s\n", mln);
     }
-    (void)fclose(mlog);			/* don't need this anymore */
+    (void) fclose(mlog);		/* don't need this anymore */
 }
 
 /*
