@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:lookup.c,v 1.44 1999/02/03 05:53:20 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:lookup.c,v 1.45 1999/09/03 21:37:42 woods Exp"
  */
 
 /*
@@ -49,8 +49,9 @@
 #endif	/* not POSIX_OS */
 
 #include "smail.h"
-#include "lookup.h"
 #include "dys.h"
+#include "log.h"
+#include "lookup.h"			/* XXX only really needed with LIBWHOSON for now */
 #include "exitcodes.h"
 
 #ifdef HAVE_NIALIAS
@@ -102,6 +103,10 @@ static int	regerr = 0;		/* error code from RE compiler */
 # endif
 #endif
 
+#ifdef HAVE_LIBWHOSON
+# include <whoson.h>
+#endif
+
 #ifdef ANSI_C
 # define VOLATILE	static volatile
 #else
@@ -1820,15 +1825,15 @@ nisplus_lookup(db, key, value, error)
  * Returns 1 if matched, else 0.
  */
 int
-match_ip(ip, pattern)
+match_ip(ip, patterns)
     char *ip;			/* ip address we are matching */
-    char *pattern;		/* pattern we are using to match it */
+    char *patterns;		/* list of patterns we are using to match it */
 {
     char	*pat_p, *end_p, wild = 0;
     int		len;
     
     /* run through the pattern list */
-    for (pat_p = pattern; /* NOTEST */ ; pat_p = end_p + 1, wild = 0) {
+    for (pat_p = patterns; /* NOTEST */ ; pat_p = end_p + 1, wild = 0) {
 
 	/* skip any spaces at front */
 	while (*pat_p && isspace(*pat_p))
@@ -1859,19 +1864,43 @@ match_ip(ip, pattern)
 		}
 		/* have match if pat was wild or we checked the whole IP */
 		if (len && strncmp(ip, smtp_local_net, len) == 0 && (wild || ip[len] == '\0')) {
-		    DEBUG3(DBG_ADDR_HI,
+		    DEBUG3(DBG_ADDR_MID,
 			   "ip '%s' verified by localnet '%s', up to len %d\n",
 			   ip, smtp_local_net, len);
 		    return 1;
 		}
+#ifdef HAVE_LIBWHOSON
+	    } else if (len > 0 && strncmpic("whoson", pat_p, len) == 0) {
+		int wso;
+		char retbuf[BUFSIZ];
+
+		DEBUG1(DBG_ADDR_HI,
+		       "(checking if '%s' is listed in the 'whoson' database)\n", ip);
+		wso = wso_query(ip, retbuf, sizeof(retbuf));
+		retbuf[sizeof(retbuf)-1] = '\0'; /* just in case... */
+		if (wso < 0) {
+		    /* XXX hope this doesn't flood the log! */
+		    write_log(WRITE_LOG_PANIC, "wso_query(%s): whoson query failed: %s\n", ip, retbuf);
+		    /* continue with next pattern */
+		} else if (wso == 0) {
+		    DEBUG2(DBG_ADDR_MID, "ip '%s' verified by whoson as '%s'\n", ip, retbuf);
+		    return 1;
+		} else {
+		    DEBUG1(DBG_ADDR_MID, "ip '%s' NOT verified by whoson\n", ip);
+		    /* continue with next pattern */
+		}
+#endif
 	    } else {
+		DEBUG3(DBG_ADDR_HI,
+		       "(checking if '%s' matches pattern '%s' to len '%d')\n",
+		       ip, pat_p, len);
 		/* We have a match if pat was wild or we checked the whole IP.
 		 * NOTE:  len may be zero for pattern of just "*" which will
 		 * mean the first term of the expression will always be true,
 		 * which is what we want in this case.
 		 */
 		if (strncmp(ip, pat_p, len) == 0 && (wild || ip[len] == '\0')) {
-		    DEBUG3(DBG_ADDR_HI,
+		    DEBUG3(DBG_ADDR_MID,
 			   "ip '%s' verified by sub-pattern of '%s', up to len %d\n",
 			   ip, pat_p, len);
 		    return 1;
@@ -1882,8 +1911,8 @@ match_ip(ip, pattern)
 	    break;
     }
 
-    DEBUG2(DBG_ADDR_HI, "ip '%s' not verified by pattern '%s'\n",
-	   ip, pattern);
+    DEBUG2(DBG_ADDR_HI, "ip '%s' not verified by any pattern in '%s'\n",
+	   ip, patterns);
     
     return 0;
 }
