@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:modes.c,v 1.82 1999/02/03 04:10:09 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:modes.c,v 1.84 1999/07/22 05:10:22 woods Exp"
  */
 
 /*
@@ -973,13 +973,13 @@ daemon_mode()
 
     /* XXX: optval argument is (const char *) on Solaris-2 and (void *) in Net/3 */
     if (setsockopt(ls, SOL_SOCKET, SO_REUSEADDR, (char *)&optval, sizeof(optval)) < 0) {
-	write_log(WRITE_LOG_SYS, "SO_REUSEADDR on failed: %s\n", strerror(errno));
+	write_log(WRITE_LOG_SYS, "SO_REUSEADDR on failed: %s.", strerror(errno));
 	exitvalue = EX_OSERR;
 	return;
     }
 
     if (bind(ls, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
-	write_log(WRITE_LOG_SYS, "bind() failed: %s", strerror(errno));
+	write_log(WRITE_LOG_SYS, "bind() failed: %s.", strerror(errno));
 	exitvalue = EX_OSERR;
 	return;
     }
@@ -1139,10 +1139,13 @@ daemon_mode()
 
 		DEBUG1(DBG_MAIN_MID, "rejecting SMTP connection #%d...\n",
 		       smtp_accept_count + 1);
+		/* use write(2) because if fdopen() failed we'd have to anyway! */
 		(void) write(as, reject[0], strlen(reject[0]));
 		(void) write(as, primary_name, strlen(primary_name));
 		(void) write(as, reject[1], strlen(reject[1]));
 		(void) close(as);
+		/* XXX hope this doesn't get too noisy! */
+		write_log(WRITE_LOG_SYS, "[%d] connection %d deferred, Too many connections!", getpid(), smtp_accept_count + 1);
 		continue;
 	    }
 #endif	/* SIGCHLD */
@@ -1155,20 +1158,20 @@ daemon_mode()
 #endif
 
 	if (got_sighup) {
-	    write_log(WRITE_LOG_SYS, "pid %d: SIGHUP received, exec(%s)",
+	    write_log(WRITE_LOG_SYS, "[%d] SIGHUP received, exec(%s)",
 		      getpid(), smail);
 	    execv(smail, save_argv);
-	    panic(EX_UNAVAILABLE, "pid %d: exec of %s failed", getpid(), smail);
+	    panic(EX_UNAVAILABLE, "[%d] exec of %s failed", getpid(), smail);
 	}
 	if (got_sigalrm) {
 	    /* if config file have changed, recycle */
 	    DEBUG(DBG_MAIN_LO, "SIGALRM received, check input queue\n");
 	    if (is_newconf()) {
 		/* re-exec smail */
-		write_log(WRITE_LOG_SYS, "pid %d: new config files, exec(%s)",
+		write_log(WRITE_LOG_SYS, "[%d] new config files, exec(%s)",
 			  getpid(), smail);
 		execv(smail, save_argv);
-		panic(EX_UNAVAILABLE, "pid %d: exec of %s failed", getpid(), smail);
+		panic(EX_UNAVAILABLE, "[%d] exec of %s failed", getpid(), smail);
 		/*NOTREACHED*/
 	    }
 	    /* reopen the log files so that they can be moved and removed */
@@ -1237,9 +1240,9 @@ do_daemon_accept(ls, fd, from)
 	int oerrno = errno;
 	FILE *f = fdopen(fd, "w");	/* XXX error check this? */
 
-	DEBUG3(DBG_MAIN_MID, "fork() failed for pid %d: rejecting SMTP connection #%d: %s\n",
+	DEBUG3(DBG_MAIN_MID, "fork() failed for PID %d: rejecting SMTP connection #%d: %s\n",
 	       getpid(), smtp_accept_count + 1, strerror(oerrno));
-	write_log(WRITE_LOG_SYS, "pid %d: fork() failed: %s", getpid(), strerror(oerrno));
+	write_log(WRITE_LOG_SYS, "[%d] fork() failed: %s", getpid(), strerror(oerrno));
 	(void) fprintf(f, "421 %s Connection refused -- system resources exausted\r\n",
 		       primary_name);
 	(void) fflush(f);
@@ -1249,6 +1252,13 @@ do_daemon_accept(ls, fd, from)
 	FILE *in;			/* input channel */
 	FILE *out;			/* output channel */
 
+	if (debug_pause_for_smtp_connections) {
+	    write_log(WRITE_LOG_SYS, "[%d] remote: pausing %d seconds for debugger startup....",
+		      getpid(),
+		      debug_pause_for_smtp_connections);
+	    sleep(debug_pause_for_smtp_connections); /* XXX what about interactions with SIGALRM? */
+	}
+
 	(void) close(ls);
 	/* setup the channels */
 	in = fdopen(fd, "r");		/* XXX error check this? */
@@ -1325,8 +1335,12 @@ static void
 daemon_sighup(sig)
     int sig;
 {
-    got_sighup = TRUE;
+    /*
+     * XXX note that pause() won't re-awaken on SIGHUP -- should we use
+     * JUMP_LONGJMP()?  Would it work?
+     */
     (void) signal(SIGHUP, daemon_sighup);
+    got_sighup = TRUE;
 }
 
 /*
@@ -1337,7 +1351,7 @@ static void
 daemon_sigterm(sig)
     int sig;
 {
-    write_log(WRITE_LOG_SYS, "pid %d: SIGTERM received, shutting down daemon.", getpid());
+    write_log(WRITE_LOG_SYS, "[%d] SIGTERM received, shutting down daemon.", getpid());
     exit(EX_OK);
 }
 
@@ -1448,13 +1462,13 @@ noop_mode()
 	/* get an alarm at intervals */
 	(void) alarm(queue_interval);
 	for (;;) {
-	    pause();
+	    pause();	/* XXX some system don't seem to awaken on SIGHUP! */
 	    /* watch for SIGHUP to indicate a recycle */
 	    if (got_sighup) {
-		write_log(WRITE_LOG_SYS, "pid %d: SIGHUP received, exec(%s)",
+		write_log(WRITE_LOG_SYS, "[%d]: SIGHUP received, exec(%s)",
 		       getpid(), smail);
 		execv(smail, save_argv);
-		panic(EX_UNAVAILABLE, "pid %d: exec of %s failed", getpid(), smail);
+		panic(EX_UNAVAILABLE, "[%d] exec of %s failed", getpid(), smail);
 		/*NOTREACHED*/
 	    }
 	    if (! got_sigalrm) {
@@ -1466,10 +1480,10 @@ noop_mode()
 
 	    /* if config file have changed, recycle */
 	    if (is_newconf()) {
-		write_log(WRITE_LOG_SYS, "pid %d: new config files, exec(%s)",
+		write_log(WRITE_LOG_SYS, "[%d] new config files, exec(%s)",
 		       getpid(), smail);
 		execv(smail, save_argv);
-		panic(EX_UNAVAILABLE, "pid %d: exec of %s failed", getpid(), smail);
+		panic(EX_UNAVAILABLE, "[%d] exec of %s failed", getpid(), smail);
 		/*NOTREACHED*/
 	    }
 	    /* reopen the log files so that they can be moved and removed */
@@ -1560,7 +1574,7 @@ start_daemon()
 
     if (queue_interval > 0) {
 	daemon_pid = getpid();
-	write_log(WRITE_LOG_SYS, "pid %d: smail daemon started", daemon_pid);
+	write_log(WRITE_LOG_SYS, "[%d] smail daemon started", daemon_pid);
     }
 
     /* grab the real uid under which smail was executed */
@@ -1750,6 +1764,10 @@ do_run_queue()
 
     DEBUG(DBG_MAIN_MID, "do_run_queue: called\n");
 
+    /*
+     * This exceedingly silly hack of special-casing the handling of a single
+     * queue entry is here to make debugging easier....
+     */
     if (num_recipients == 1 && recipients) {
 	/*
 	 * If it wasn't for the fact that there might be more than one spool
@@ -1764,20 +1782,24 @@ do_run_queue()
 			      program, recipients->in_addr);
 		(void)fflush(errfile);
 	    }
+	} else {
+	    process_one_spool_file(*work);
 	}
-	process_one_spool_file(*work);
 	DEBUG(DBG_MAIN_HI, "do_run_queue: finished processing one queue entry\n");
         return;
     }
 
-    /* get work, and do it with child processes */
+    /*
+     * If we're either handling all the queue entries, or several queue
+     * entries, then build the work list, and do it with child processes....
+     */
     work = scan_spool_dirs();
     while (*work) {
 	if (errfile) {
 	    (void) fflush(errfile);
 	}
 	if (process_spool_file(*work) == FAIL) {
-	    /* fork failed, don't continue */
+	    /* fork failed, error logged in subroutine, don't continue */
 	    return;
 	}
 
@@ -2119,12 +2141,12 @@ process_spool_file(spfn)
     int pid;
     struct stat statbuf;
 
-    DEBUG1(DBG_MAIN_MID, "process_spool_file(%s): called\n", spfn);
+    DEBUG1(DBG_MAIN_LO, "process_spool_file(%s): called\n", spfn);
     /*
      * Verify the spool file exists before we try to process it
      */
     if (stat(spfn, &statbuf) < 0) {
-	DEBUG2(DBG_MAIN_MID, "process_spool_file: %s: File possibly already processed: %s",
+	DEBUG2(DBG_MAIN_MID, "process_spool_file: %s: File possibly already processed: %s\n",
 	      spfn, strerror(errno));
 	return SUCCEED;			/* we only fail if fork_wait() fails!  */
     }
@@ -2168,6 +2190,8 @@ process_one_spool_file(spfn)
     /* in child process */
     char **argv;		/* arguments from spool file */
 
+    DEBUG1(DBG_MAIN_LO, "process_one_spool_file(%s): called\n", spfn);
+
     /* message grade is encoded in the last char of the filename */
     msg_grade = spfn[strlen(spfn) - 1];
 
@@ -2177,7 +2201,7 @@ process_one_spool_file(spfn)
     /* attempt to open the message */
     if (open_spool(spfn, TRUE, WRITE_LOG_SYS) == FAIL) {
 	if (exitvalue == EX_TEMPFAIL || exitvalue == EX_NOINPUT) {
-	    DEBUG2(DBG_MAIN_LO, "process_one_spool_file: %s: file %s", spfn,
+	    DEBUG2(DBG_MAIN_LO, "process_one_spool_file: %s: file %s\n", spfn,
 		   (exitvalue == EX_TEMPFAIL) ? "already locked; skipping" : "not found (already processed?)");
 	} else {
 	    panic(exitvalue, "process_one_spool_file(): could not process %s -- see log.", spfn);
