@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:notify.c,v 1.32 1999/03/31 08:08:35 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:notify.c,v 1.34 1999/08/05 02:50:17 woods Exp"
  */
 
 /*
@@ -652,8 +652,8 @@ notify(defer, fail, sent_errors)
     }
 
     /*
-     * for failed addresses put them in two lists, one list going to
-     * the sender or the postmaster, the other list going to addr owners.
+     * put failed addresses in two lists, one list going to the sender or the
+     * postmaster, the other list of addr owners to be verified.
      */
     for (cur = fail; cur; cur = next) {
 	next = cur->succ;
@@ -693,7 +693,7 @@ notify(defer, fail, sent_errors)
 	if (error_processing != MAIL_BACK && error_processing != WRITE_BACK) {
 	    return_to_sender = FALSE;
 	}
-	/* turn it off, as well, for low priority messages */
+	/* turn it off, as well, for really low priority messages */
 	if (islower(msg_grade) && msg_grade >= 'n') {
 	    return_to_sender = FALSE;
 	}
@@ -721,6 +721,7 @@ notify(defer, fail, sent_errors)
 	! send_to_postmaster &&
 	to_owner == NULL)
     {
+	DEBUG(DBG_NOTIFY_LO, "notify: nothing to do.\n");
 	return;
     }
 
@@ -758,20 +759,25 @@ notify(defer, fail, sent_errors)
 			   "sending to address owner");
 	send_log(f, FALSE, log_banner);	/* don't include X lines from log */
 	(void) fputs(addr_error_banner, f);
+	(void) fprintf(f, "Owner address: %s.\n", cur->in_addr);
+	last_owner = cur->in_addr;
 	/* write out all of the address errors for messages with the same owner */
 	do {
-	    struct addr *ta = cur;
-
-	    if (ta->parent)
-		ta = ta->parent;
-	    if (ta->true_addr && ta->true_addr->error) {
-		(void) fprintf(f, " %s ... failed: %s\n", ta->true_addr->in_addr, ta->true_addr->error->message);
-	    } else if (!ta->true_addr->error) {
-		panic(EX_SOFTWARE, "notify(): no error info for true_addr->in_addr=%s", ta->true_addr->in_addr);
-	    } else if (!ta->true_addr) {
-		panic(EX_SOFTWARE, "notify(): no true_addr for in_addr=%s", ta->in_addr);
+	    if (cur->parent && cur->parent->error) {
+		(void) fprintf(f, " parent addr %s ... failed: %s\n",
+			       cur->parent->in_addr ? cur->parent->in_addr : "(unknown)",
+			       cur->parent->error->message);
+	    }
+	    if (cur->true_addr && cur->true_addr->error) {
+		(void) fprintf(f, " true_addr %s ... failed: %s\n",
+			       cur->true_addr->in_addr ? cur->true_addr->in_addr : "(unknown)",
+			       cur->true_addr->error->message);
+	    }
+	    if (cur->error) {
+		(void) fprintf(f, " %s ... failed: %s\n",
+			       cur->in_addr ? cur->in_addr : "(unknown)",
+			       cur->error->message);
 	    }
-	    last_owner = cur->in_addr;
 	    cur = cur->succ;
 	} while (cur && EQIC(last_owner, cur->in_addr));
 	(void) fputs(text_banner, f);
@@ -792,14 +798,20 @@ notify(defer, fail, sent_errors)
 	/* write messages in to_other destined to the postmaster */
 	(void) fputs(addr_error_banner, f);
 	for (cur = to_other; cur; cur = cur->succ) {
-	    struct addr *ta = cur;
-
-	    if (ta->parent)
-		ta = ta->parent;
-	    if (ta->error) {		/* XXX should use true_addr too???? */
-		(void) fprintf(f, " %s ... %s\n", ta->in_addr, ta->error->message);
-	    } else {
-		panic(EX_SOFTWARE, "notify(send_to_postmaster): no error info for in_addr=%s", ta->in_addr);
+	    if (cur->parent && cur->parent->error) {
+		(void) fprintf(f, " parent addr %s ... failed: %s\n",
+			       cur->parent->in_addr ? cur->parent->in_addr : "(unknown)",
+			       cur->parent->error->message);
+	    }
+	    if (cur->true_addr && cur->true_addr->error) {
+		(void) fprintf(f, " true_addr %s ... failed: %s\n",
+			       cur->true_addr->in_addr ? cur->true_addr->in_addr : "(unknown)",
+			       cur->true_addr->error->message);
+	    }
+	    if (cur->error) {
+		(void) fprintf(f, " %s ... failed: %s\n",
+			       cur->in_addr ? cur->in_addr : "(unknown)",
+			       cur->error->message);
 	    }
 	}
 	(void) fputs(text_banner, f);
@@ -820,14 +832,20 @@ notify(defer, fail, sent_errors)
 	/* write messages in to_other destined to the sender */
 	(void) fputs(addr_error_banner, f);
 	for (cur = to_other; cur; cur = cur->succ) {
-	    struct addr *ta = cur;
-
-	    if (ta->parent)
-		ta = ta->parent;
-	    if (ta->error) {		/* XXX should use true_addr too???? */
-		(void) fprintf(f, " %s ... %s\n", ta->in_addr, ta->error->message);
-	    } else {
-		panic(EX_SOFTWARE, "notify(return_to_sender): no error info for in_addr=%s", ta->in_addr);
+	    if (cur->parent && cur->parent->error) {
+		(void) fprintf(f, " parent addr %s ... failed: %s\n",
+			       cur->parent->in_addr ? cur->parent->in_addr : "(unknown)",
+			       cur->parent->error->message);
+	    }
+	    if (cur->true_addr && cur->true_addr->error) {
+		(void) fprintf(f, " true_addr %s ... failed: %s\n",
+			       cur->true_addr->in_addr ? cur->true_addr->in_addr : "(unknown)",
+			       cur->true_addr->error->message);
+	    }
+	    if (cur->error) {
+		(void) fprintf(f, " %s ... failed: %s\n",
+			       cur->in_addr ? cur->in_addr : "(unknown)",
+			       cur->error->message);
 	    }
 	}
 	(void) fputs(text_banner, f);
@@ -838,6 +856,9 @@ notify(defer, fail, sent_errors)
 	 * This is handy when notification of errors is desired, but
 	 * when the overhead of sending the entire message back is not
 	 * necessary.
+	 *
+	 * Note that grades >= 'n' have the return_to_sender flag turned off
+	 * completely above.
 	 */
 	if (islower(msg_grade)) {
 	    fputs("[low-priority message, body not included]\n", f);
@@ -1145,10 +1166,10 @@ hash_predelivered_addresses()
 
 	    /* Find if  address is non-local */
 	    if (*preparsed_address != '|') { /* Ensure piped addresses are stripped out */
-		work_address.i = 0;
+		STR_CLEAR(&work_address);
 		STR_CAT(&work_address, preparsed_address);
 		parse_flag = 0;
-		if ((parse_address(work_address.p, &target, &remainder, &parse_flag) != LOCAL) 
+		if ((parse_address(STR(&work_address), &target, &remainder, &parse_flag) != LOCAL) 
 		    && (!islocalhost(target))) {
 		    DEBUG1(DBG_NOTIFY_LO, "Hashing out %s ... previously processed.\n", address);
 		    (void) add_to_hash(address, (char *)NULL, 0, hit_table);
