@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:queue.c,v 1.38 1998/09/21 05:31:46 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:queue.c,v 1.40 1999/07/22 05:11:40 woods Exp"
  */
 
 /*
@@ -189,7 +189,7 @@ queue_envelope(list)
 {
     register struct addr *cur;		/* current recipient addr to process */
     char *p;				/* temp */
-    char buf[50];			/* temp storage */
+    char buf[128];			/* temp storage (> MAXUSER and > "%d %d" */
 
     if (local_sender) {
 	if (put_line(local_sender) == EOF)
@@ -627,10 +627,10 @@ read_message()
 
     /* read the real uid and effective gid under which the mailer was exec'd */
     if (c != READ_FAIL && c != EOF) {
-	char buf[50];			/* temp input buffer */
+	char buf[128];			/* temp input buffer */
 	char *p2 = buf;			/* temp for scanning buf */
 
-	while ((c = GETSPOOL()) != '\n' && c != EOF && c != READ_FAIL) {
+	while ((c = GETSPOOL()) != '\n' && c != EOF && c != READ_FAIL && (p2-buf) < sizeof(buf)) {
 	    *p2++ = c;
 	}
 	*p2 = '\0';
@@ -1294,11 +1294,14 @@ scan_spool_dirs()
 	char *fn;			/* filename from directory */
 	char *in_dn = xprintf("%s/input", dn);
 
+	DEBUG1(DBG_QUEUE_HI, "scan_spool_dirs() looking in %s\n", in_dn);
+
 	for (fn = scan_dir(in_dn); fn; fn = scan_dir((char *)NULL)) {
 	    int entry_grade;
 
 	    /* see if the file matches the form of a spool file */
 	    if (! isdigit(fn[0]) || strlen(fn) != SPOOL_FN_LEN) {
+		DEBUG1(DBG_QUEUE_MID, "scan_spool_dirs() ignoring %s\n", fn);
 		continue;		/* nope */
 	    }
 	    /*
