@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:resolve.c,v 1.14 1998/11/20 00:31:58 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:resolve.c,v 1.15 1999/07/22 05:12:50 woods Exp"
  */
 
 /*
@@ -141,26 +141,19 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
 		 *      resubmit the message.
 		 */
 		cur->error = note_error(ERR_NSOWNER|ERR_111, cur->remainder);
-		cur->flags |= PARSE_ERROR;
+		cur->flags &= ~ADDR_FORM_MASK;
 		cur->succ = *fail;
 		*fail = cur;
 		continue;
 
 	    case LOCAL:
-		/*
-		 * a local-form in quotes must be reparsed
-		 */
-		if (strip(cur->remainder)) {
-		    /* it was in quotes, put it back on the input */
-		    DEBUG1(DBG_RESOLVE_HI, "re-parsing quoted local form %s.\n", cur->remainder);
-		    next = cur;
-		} else {
-		    if (!cur->local_name)
-			cur->local_name = visible_name;
-		    cur->succ = local;
-		    local = cur;
-		    DEBUG1(DBG_RESOLVE_HI, "moved local form %s to be directed.\n", cur->remainder);
+		(void) strip(cur->remainder); /* strip any quotes, etc. */
+		if (!cur->local_name) {
+		    cur->local_name = visible_name;
 		}
+		cur->succ = local;
+		local = cur;
+		DEBUG1(DBG_RESOLVE_HI, "moved local form %s to be directed.\n", cur->remainder);
 		break;
 
 	    default:			/* anything else is a remote-form address */
