@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:route.c,v 1.31 1999/02/03 06:27:42 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:route.c,v 1.32 1999/06/26 16:43:33 woods Exp"
  */
 
 /*
@@ -37,6 +37,7 @@
 #endif /* HAVE_BSD_NETWORKING */
 #include "transport.h"
 #include "addr.h"
+#include "field.h"
 #include "dys.h"
 #include "log.h"
 #include "exitcodes.h"
@@ -182,20 +183,64 @@ route_remote_addrs(in, out, retry, defer, fail)
 	    continue;
 	}
 	if (cur->next_host == NULL) {
+	    char *value;		/* copy of cur->remainder */
+	    char *field_error = NULL;
+	    struct addr *list = NULL;	/* list returned by parse_field() */
+	    struct addr *next_list;	/* iterator */
+
 	    /* router says to reparse */
 	    DEBUG2(DBG_ROUTE_LO, "    routed %s --> %s for re-parsing.\n",
 		   (cur->parent) ? cur->parent->in_addr : cur->in_addr, cur->remainder);
-	    cur->work_addr = COPY_STRING(cur->remainder);/* XXX will leak? */
-	    cur->flags &= ~(ADDR_FINISHED | ADDR_FULLMATCH);
-	    cur->transport = NULL;
-	    cur->tphint_list = NULL;
-	    if (cur->route) {
-		xfree(cur->route);
-		cur->route = NULL;
+
+	    value = COPY_STRING(cur->remainder); /* needed for process_field()  */
+	    /*
+	     * the "address" to re might be a whole list of addresses from a
+	     * rewrite lookup in an aliase-style database, etc., so we must
+	     * parse it appart and create a new list of addresses for the retry
+	     */
+	    (void) process_field((char *) NULL, value, (char *) NULL,
+				 (char *) NULL, &list, F_ALIAS, &field_error);
+	    if (field_error) {
+		DEBUG1(DBG_ROUTE_HI, "     cannot parse field: %s\n", field_error);
+		/*
+		 * ERR_111 - malformed address(es) returned by the router
+		 *
+		 * DESCRIPTION
+		 *      The address(es) returned for re-parsing choked
+		 *      process_field().  This is probably a result of a broken
+		 *      value returned by the rewrite router from its database.
+		 *
+		 * ACTIONS
+		 *      The message is deferred due to the configuration error.
+		 *
+		 * RESOLUTION
+		 *      The postmaster must fix the source of the bad address.
+		 */
+		cur->error = note_error(ERR_CONFERR|ERR_111,
+					xprintf("router %s: driver %s: routed <%s> to un-parsable field '%s': %s",
+						cur->router->name,
+						cur->router->driver,
+						(cur->parent) ? cur->parent->in_addr : cur->in_addr,
+						cur->remainder,
+						field_error));
+		cur->succ = *fail;
+		*fail = cur;
+		if (*retry)
+		    xfree(*retry);
+		*retry = NULL;
+	    } else {
+		    for (; list; list = next_list) {
+			    next_list = list->succ;
+			    
+			    list->parent = cur;
+			    list->owner = NULL;
+			    list->director = cur->director; /* XXX? */
+			    list->router = cur->router; /* XXX? */
+			    list->flags = ADDR_ALIASTYPE;
+			    list->succ = *retry;
+			    *retry = list;
+		    }
 	    }
-	    cur->remainder = NULL;
-	    cur->succ = *retry;
-	    *retry = cur;
 	    continue;
 	}
 	if (cur->transport == NULL) {
