@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_106:gethost.c,v 1.17 1998/11/20 01:05:27 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_107:gethost.c,v 1.18 1999/09/08 03:52:26 woods Exp"
  */
 
 /*
@@ -103,6 +103,14 @@
 # define INADDR_LOOPBACK	((unsigned long) 0x7F000001)
 #endif
 
+/*
+ * import h_errno; many systems don't define it in <netdb.h>
+ */
+
+#ifndef OBSOLETE_RESOLVER
+extern int h_errno;
+#endif
+
 /* functions local to this file */
 static int gethostbyname_lookup();
 static int gethostbyaddr_lookup();
