@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:smailconf.c,v 1.40 1998/10/07 05:25:53 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:smailconf.c,v 1.43 1999/09/03 21:41:17 woods Exp"
  */
 
 /*
@@ -44,6 +44,7 @@
 #define M_MULTIPLIER	(1024*1024)
 
 /* functions local to this file */
+static int format_attribute();
 static void print_config_help();
 static void print_config_all();
 
@@ -91,6 +92,7 @@ static struct attr_table conf_attributes[] = {
     { "hostnames", t_string, NULL, (tup *)&hostnames, 0 },
     { "hostname", t_string, NULL, (tup *)&hostnames, 0 },
     { "listen_name", t_string, NULL, (tup *)&listen_name, 0 },
+    { "sending_name", t_string, NULL, (tup *)&sending_name, 0 },
     { "lock_by_name", t_boolean, NULL, (tup *)&lock_by_name, 0 },
     { "lock_mode", t_mode, NULL, (tup *)&lock_mode, 0 },
     { "log_mode", t_mode, NULL, (tup *)&log_mode, 0 },
@@ -147,6 +149,7 @@ static struct attr_table conf_attributes[] = {
     { "smtp_hello_verify_ptr", t_boolean, NULL, (tup *)&smtp_hello_verify_ptr, 0 },
     { "smtp_sender_verify_mx_only", t_boolean, NULL, (tup *)&smtp_sender_verify_mx_only, 0 },
     { "smtp_max_recipients", t_int, NULL, (tup *)&smtp_max_recipients, 0 },
+    { "debug_pause_for_smtp_connections", t_int, NULL, (tup *)&debug_pause_for_smtp_connections, 0},
     { "spool_dirs", t_string, NULL, (tup *)&spool_dirs, 0 },
     { "spool_dir", t_string, NULL, (tup *)&spool_dir, 0 },
     { "spool_grade", t_char, NULL, (tup *)&spool_grade, 0 },
@@ -174,7 +177,8 @@ struct attr_table *end_conf_attributes = ENDTABLE(conf_attributes);
 
 
 /*
- * read_config_file - read a config file and fill in conf_attributes
+ * read_config_file - read a config file, or a colon separated list of config
+ * files, and fill in conf_attributes
  *
  * return an error message or NULL (if no error).
  */
@@ -190,9 +194,38 @@ read_config_file(fn)
 	/* a name of `-' turns off use of the external file */
 	return NULL;
     }
+    /* 
+     * iterate through each file in succession if multiple names are given,
+     * separated by colons, and recursively read and process them....
+     */
+    if (strchr(fn, ':')) {
+	char *p_fn, *p_nextfn, *p_start, *p_end, *p_result;
 
-    f = fopen(fn, "r");
-    if (f == NULL) {
+	p_fn = COPY_STRING(fn);
+	for (p_start = p_fn; (p_end = strchr(p_start, ':')); p_start = p_end + 1) {
+	    *p_end = '\0';
+	    p_nextfn = make_lib_fn(p_start);
+	    p_result = read_config_file(p_nextfn);
+	    if (p_nextfn)
+		xfree(p_nextfn);
+	    if (p_result) {
+		if (p_fn)
+		    xfree(p_fn);
+		return(p_result);	/* pop any errors back out */
+	    }
+	}
+	p_result = NULL;
+	if (p_start && *p_start) {
+	    p_nextfn = make_lib_fn(p_start);
+	    p_result = read_config_file(p_nextfn);
+	    if (p_nextfn)
+		xfree(p_nextfn);
+	}
+	if (p_fn)
+	    xfree(p_fn);
+	return(p_result);
+    }
+    if (! (f = fopen(fn, "r"))) {
 	if (require_configs) {
 	    return xprintf("%s: %s", fn, strerror(errno));
 	}
@@ -201,7 +234,7 @@ read_config_file(fn)
 	return NULL;
     }
 
-    (void)fstat(fileno(f), &statbuf);
+    (void) fstat(fileno(f), &statbuf);
     add_config_stat(fn, &statbuf);
 
     /*
@@ -412,7 +445,7 @@ quote_string_value(str)
  * Return value is bit mask which items are valid
  */
 
-int
+static int
 format_attribute(conf_attr, name, quoted, prefix, value, ptr, flags)
      struct attr_table * conf_attr;	/* The attriute info */
      char * name;			/* The conf variable name */
