@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:spool.c,v 1.26 1998/04/11 23:30:56 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:spool.c,v 1.27 1999/07/22 05:14:52 woods Exp"
  */
 
 /*
@@ -1128,8 +1128,10 @@ close_spool()
 	       spool_dir, lock_fn);
 	(void) unlink(lock_fn);
     }
-    (void) close(spoolfile);
-    spoolfile = -1;
+    if (spoolfile != -1) {
+	(void) close(spoolfile);
+	spoolfile = -1;
+    }
     spool_fn = NULL;
     input_spool_fn = NULL;
 }
@@ -1140,17 +1142,14 @@ close_spool()
 void
 unlink_spool()
 {
-    char *save_input_spool_fn = input_spool_fn;
-
     DEBUG(DBG_SPOOL_HI, "unlink_spool called\n");
-    if (message_date() == time((time_t *) NULL)) {
-	DEBUG(DBG_SPOOL_HI, "pausing to prevent spool file clash\n");
-	sleep(2);
-    }
-    if (save_input_spool_fn) {
-	DEBUG2(DBG_SPOOL_HI, "unlink_spool: unlinking: %s/%s\n",
-	       spool_dir, save_input_spool_fn);
-	(void) unlink(save_input_spool_fn);
+    if (input_spool_fn) {
+	if (message_id && message_date() == time((time_t *) NULL)) {
+	    DEBUG(DBG_SPOOL_MID, "unlink_spool: pausing to prevent spool file clash\n");
+	    sleep(2);
+	}
+	DEBUG2(DBG_SPOOL_HI, "unlink_spool: unlinking: %s/%s\n", spool_dir, input_spool_fn);
+	(void) unlink(input_spool_fn);
     }
     close_spool();
 }
@@ -1650,11 +1649,14 @@ new_grade(grade)
 time_t
 message_date()
 {
-    time_t clck;
+    time_t clck = 0;
     register int digit;
     register char *p;
 
-    clck = 0;
+    if (!message_id) {
+	DEBUG(DBG_SPOOL_LO, "message_date(): called with no message_id set!\n");
+	return clck;
+    }
     for (p = message_id + 1; *p; p++) {
 	if (isdigit(*p)) {
 	    digit = *p - '0';
