@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:string.c,v 1.25 1998/11/06 20:15:11 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:string.c,v 1.26 1999/07/19 01:45:18 woods Exp"
  */
 
 /*
@@ -574,7 +574,6 @@ base62(val)
     return buf;
 }
 
-#ifdef notused
 /*
  * read_line - read a line from an input stream and return a pointer to it
  *
@@ -594,15 +593,14 @@ read_line(f)
 	inited = TRUE;
 	STR_INIT(&line);
     } else {
-	line.i = 0;
+	STR_CLEAR(&line);
     }
     while ((c = getc(f)) != '\n' && c != EOF) {
 	STR_NEXT(&line, c);
     }
     STR_NEXT(&line, '\0');
-    return line.p;
+    return STR(&line);
 }
-#endif	/* notused */
 
 /*
  * str_cat - concatenate a C string onto the end of a dynamic string
