@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_106:sysdep.c,v 1.74 1998/11/20 02:21:50 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_107:sysdep.c,v 1.77 1999/08/05 02:33:41 woods Exp"
  */
 
 /*
@@ -256,8 +256,8 @@ extern int errno;
 char *
 time_stamp()
 {
-    time_t sclock = time((time_t *) NULL);
-    struct tm *ltm = localtime(&sclock);
+    time_t curtime = time((time_t *) NULL);
+    struct tm *ltm = localtime(&curtime);
     static char timebuf[sizeof("mm/dd/yyyy hh:mm:ss")];
 
     (void) sprintf(timebuf, "%02d/%02d/%04d %02d:%02d:%02d",
@@ -292,8 +292,8 @@ get_local_year()
  * get_arpa_date - return date in RFC822 format
  */
 char *
-get_arpa_date(clock)
-    time_t clock;
+get_arpa_date(sometime)
+    time_t sometime;
 {
     struct tm gtm, *ltm;
     char *tzinfo;
@@ -303,9 +303,9 @@ get_arpa_date(clock)
      * We need to do this since most implementations will use the same static
      * buffer for both localtime and gmtime 
      */
-    ltm = gmtime(&clock);
+    ltm = gmtime(&sometime);
     (void) memcpy(&gtm, ltm, sizeof(gtm));
-    ltm = localtime(&clock);
+    ltm = localtime(&sometime);
     tzinfo = get_time_zone(&gtm, ltm);
 
     (void) sprintf(timebuf, "%s, %d %s %04d %02d:%02d:%02d %s",
@@ -396,8 +396,8 @@ struct tm *ltm;
 char *
 unix_date()
 {
-    time_t uxclock = time((time_t *) NULL);
-    char *date = ctime(&uxclock);
+    time_t curtime = time((time_t *) NULL);
+    char *date = ctime(&curtime);
 
     /*
      * XXX - ctime format is quite standard, so just put the nul in
@@ -922,7 +922,7 @@ static void
 check_stale(l_fn)
     char *l_fn;				/* name of lock file */
 {
-    char buf[50];
+    char buf[50];			/* for contents of lock file (a PID) */
     int ct;
     int fd;
     int pid = 0;
@@ -1349,7 +1349,7 @@ compute_domain ( hostname )
  *
  * output:
  *	pid of child process, or 0 if in child process (only if argv==NULL).
- *	Return EOF if pipe() or fork() failed.
+ *	Return EOF on critical errors, such as if pipe() or fork() failed.
  */
 int
 open_child(argv, envp, in, out, errfd, flags, uid, gid)
@@ -1367,6 +1367,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
     /* remember, fd[0] is the read descriptor, fd[1] is the write descriptor */
     int retries = FORK_RETRIES;		/* countdown of retries */
     int pid;				/* pid from fork() */
+    int oerrno;				/* errno from execv*() */
     static char *min_env[] = {
 #ifdef	CHILD_ENVIRON
 	CHILD_ENVIRON,
@@ -1380,6 +1381,13 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
     };
     static char **minp = NULL;
 
+    if (!argv) {
+	write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
+		  "open_child() called with NULL argv!");
+	return EOF;
+    }
+    DEBUG5(DBG_DRIVER_HI, "open_child('%s' ... , env, in, out, %s, 0%o, %d, %d) called\n",
+	   argv[0], errfd >= 0 ? "errlog" : "noerrlog", flags, uid, gid);
     if (in && pipe(stdin_fd) < 0) {
 	write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
 		  "open_child(%s): pipe(stdin_fd) failed: %s", argv[0], strerror(errno));
@@ -1398,8 +1406,8 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 
     /* keep trying to create a fork until we succeed or retries == 0 */
     while (
-#if	(defined(UNIX_BSD) || defined(HAVE_VFORK)) && !defined(DONT_USE_VFORK)
-	   (pid = (argv ? vfork() : fork())) < 0 &&
+#if (defined(UNIX_BSD) || defined(HAVE_VFORK)) && !defined(DONT_USE_VFORK)
+	   (pid = vfork()) < 0 &&
 #else	/* not UNIX_BSD && not HAVE_VFORK */
 	   (pid = fork()) < 0 &&
 #endif	/* UNIX_BSD || HAVE_VFORK */
@@ -1410,7 +1418,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
     }
     if (pid < 0) {
 	write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
-		  "open_child(%s): fork() failed: %s", argv[0], strerror(errno));
+		  "open_child(%s): fork() failed after %d: %s", argv[0], FORK_RETRIES, strerror(errno));
 	/* free resources */
 	if (in) {
 	    (void) close(stdin_fd[0]);
@@ -1569,48 +1577,45 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 	    _exit(EX_NOPERM);
 	}
 
-	if (argv) {
-	    int oerrno;
+	/* execute the program */
+	if (envp) {
+	    (void) execve(argv[0], argv, envp);
+	    oerrno = errno;
+	    DEBUG2(DBG_DRIVER_LO, "open_child(): execve(%s, envp): %s.\n", argv[0], strerror(errno));
+	} else if (flags & CHILD_MINENV) {
+	    if (minp == NULL) {
+		/* pass through the TZ environment variable */
+		char *tz;
 
-	    /* execute the program */
-	    if (envp) {
-		(void) execve(argv[0], argv, envp);
-		oerrno = errno;
-		DEBUG2(DBG_DRIVER_LO, "open_child(): execve(%s, envp): %s.\n", argv[0], strerror(errno));
-	    } else if (flags & CHILD_MINENV) {
-		if (minp == NULL) {
-		    /* pass through the TZ environment variable */
-		    char *tz;
-
-		    for (minp = min_env; *minp; minp++)
-			;
+		for (minp = min_env; *minp; minp++) {
+		    ;
+		}
 #ifndef CHILD_ENVIRON
 # ifdef SHELL_EXEC_PATH
-		    *minp++ = xprintf("SHELL=%s", SHELL_EXEC_PATH);
+		*minp++ = xprintf("SHELL=%s", SHELL_EXEC_PATH);
 # else
-		    *minp++ = "SHELL=/bin/sh";
+		*minp++ = "SHELL=/bin/sh";
 # endif
-		    *minp++ = xprintf("PATH=%s", SECURE_PATH);
+		*minp++ = xprintf("PATH=%s", SECURE_PATH);
 #endif	/* CHILD_ENVIRON */
-		    if ((tz = getenv("TZ"))) {
-			*minp++ = xprintf("TZ=%s", tz);
-		    }
+		if ((tz = getenv("TZ"))) {
+		    *minp++ = xprintf("TZ=%s", tz);
 		}
-		(void) execve(argv[0], argv, min_env);
-		oerrno = errno;
-		DEBUG2(DBG_DRIVER_LO, "open_child(): execve(%s, minenv): %s.\n", argv[0], strerror(errno));
-	    } else {
-		(void) execv(argv[0], argv);
-		oerrno = errno;
-		DEBUG2(DBG_DRIVER_LO, "open_child(): execv(%s): %s.\n", argv[0], strerror(errno));
 	    }
-
-	    /* Oh well, all this work and we can't even exec the file */
-	    write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
-		      "open_child(): exec(%s) failed (possible local configuration error): %s",
-		      argv[0], strerror(oerrno));
-	    _exit(EX_OSFILE);
+	    (void) execve(argv[0], argv, min_env);
+	    oerrno = errno;
+	    DEBUG2(DBG_DRIVER_LO, "open_child(): execve(%s, minenv): %s.\n", argv[0], strerror(errno));
+	} else {
+	    (void) execv(argv[0], argv);
+	    oerrno = errno;
+	    DEBUG2(DBG_DRIVER_LO, "open_child(): execv(%s): %s.\n", argv[0], strerror(errno));
 	}
+
+	/* Oh well... all this work and we can't even exec the file! */
+	write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
+		  "open_child(): exec(%s) failed (possible local configuration error): %s",
+		  argv[0], strerror(oerrno));
+	_exit(EX_OSFILE);
     } else {
 	/* in parent process */
 	if (in) {
@@ -2037,7 +2042,7 @@ hstrerror(h_err)
     case NO_DATA:
 	return "No DNS data associated with name";
 
-#if (NO_ADDRESS) != (NO_DATA)
+#if defined(NO_ADDRESS) && ((NO_ADDRESS - 0) != (NO_DATA))
     case NO_ADDRESS:
 	return "No address associated with name";
 #endif
