@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_106:pipe.c,v 1.27 1998/12/16 21:42:19 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_107:pipe.c,v 1.28 1999/06/26 16:46:39 woods Exp"
  */
 
 /*
@@ -709,6 +709,7 @@ get_pipe_ugid(tp, addr, uid, gid)
     int *gid;				/* store gid here */
 {
     struct pipe_private *priv = (struct pipe_private *)tp->private;
+
     /*
      * determine the uid to use for delivery
      */
@@ -731,12 +732,24 @@ get_pipe_ugid(tp, addr, uid, gid)
 	    *uid = nobody_uid;
 	} else {
 	    *uid = pw->pw_uid;
-	    *gid = pw->pw_gid;
+	    *gid = pw->pw_gid;		/* set in case "group" attribute not given */
 	}
     }
 
-    /* determine the gid for use for delivery */
-    if (priv->group) {
+    /*
+     * determine the gid for use for delivery (if not set above)
+     */
+    if (priv->group == NULL) {
+	if (priv->user == NULL) {
+	    if ((tp->flags & PIPE_AS_SENDER)) {
+		*gid = prog_egid;
+	    } else if ((tp->flags & PIPE_AS_USER) && addr->gid != BOGUS_GROUP) {
+		*gid = addr->gid;
+	    } else {
+		*gid = nobody_gid;
+	    }
+	}
+    } else {
 	struct group *gr = getgrbyname(priv->group);
 
 	if (gr == NULL) {
@@ -748,15 +761,4 @@ get_pipe_ugid(tp, addr, uid, gid)
 	    *gid = gr->gr_gid;
 	}
     }
-    if (priv->group == NULL) {
-	if (priv->user == NULL) {
-	    if ((tp->flags & PIPE_AS_SENDER)) {
-		*gid = prog_egid;
-	    } else if ((tp->flags & PIPE_AS_USER) && addr->gid != BOGUS_GROUP) {
-		*gid = addr->gid;
-	    } else {
-		*gid = nobody_gid;
-	    }
-	}
-    }
 }
