@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_106:smtplib.c,v 1.30 1998/04/11 23:31:46 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_107:smtplib.c,v 1.32 1999/08/14 05:32:53 woods Exp"
  */
 
 /*
@@ -15,6 +15,15 @@
  *	Send mail using the SMTP protocol.  This soure file is a set
  *	of library routines that can be used by transport drivers that
  *	can handle creating the virtual circuit connections.
+ *
+ * A NOTE ABOUT SMTP ERROR LOGGING:
+ *
+ * Note that the reply from the remote server should be logged (with
+ * write_log() and/or note_error(), as appropriate) starting with a preceding
+ * newline so that entire mult-line replies will remain intact and properly
+ * formatted in the log files.  There is code in log.c send_log(), for example,
+ * which properly parses these messages in such a way that they will be treated
+ * as the single entry that they are.
  */
 #include <sys/types.h>
 #include <stdio.h>
@@ -60,10 +69,11 @@
 #define MAIL_END	">"
 #define RCPT_BEGIN	"RCPT TO:<"
 #define RCPT_END	">"
-#define DATA		"DATA"
+#define DATA_BEGIN	"DATA"
 #define DATA_END	"."
 #define QUIT		"QUIT"
 #define VERB		"VERB"
+#define CMD_RESET	"RSET"
 
 /* reply code groups, encoded in hex */
 #define POSITIVE_DEBUG		0x000	/* debugging messages */
@@ -82,13 +92,22 @@
 #define REPLY_WILLFORWARD	0x251	/* user address will be forwarded */
 #define REPLY_START_DATA	0x354	/* okay to start sending message */
 #define REPLY_DOWN		0x421	/* remote SMTP closing down */
-#define REPLY_STORAGE_FULL	0x452	/* remote storage full */
+#define REPLY_MAILBOX_UNAVAIL	0x450	/* mailbox busy */
+#define REPLY_REMOTE_FAILURE	0x451	/* remote error in processing */
+#define REPLY_STORAGE_FULL	0x452	/* remote system storage full */
+#define REPLY_COMMAND_ERROR	0x500	/* Syntax error in command */
+#define REPLY_PARAMETER_ERROR	0x501	/* Syntax error in parameter(s) */
+#define REPLY_CMD_NOT_IMPL	0x502	/* Command recognized but not implemented */
+#define REPLY_SEQUENCE_ERROR	0x503	/* Bad sequence of commands */
+#define REPLY_BAD_PARAMETER	0x504	/* Parameter recognized but not implemented */
 #define REPLY_UNKNOWN_USER	0x550	/* remote says mailbox unknown */
+#define REPLY_USER_NOT_LOCAL	0x551	/* remote says mailbox is not local */
+#define REPLY_OVER_QUOTA	0x552	/* remote says mailbox exceeded storage quota */
+#define REPLY_ILLEGAL_USER	0x553	/* remote says mailbox name not allowed */
+#define REPLY_TRANS_FAILURE	0x554	/* transaction failed (no SMTP service avail) */
 /* pseudo-reply codes */
 #define REPLY_PROTO_ERROR	0x498	/* protocol error on read */
 #define REPLY_TIMEOUT		0x499	/* timeout on read, or EOF on read */
-#define REPLY_NOT_ACCEPTABLE	0x501	/* request not acceptable */
-#define REPLY_SEQUENCE_ERROR	0x503	/* Bad sequence of commands */
 
 /* variables local to this file */
 static struct str smtp_out;		/* region for outgoing commands */
@@ -112,8 +131,13 @@ static void catch_timeout(int);
 static struct error * no_remote(struct transport *, char *);
 static struct error * try_again(struct transport *, char *);
 static struct error * fatal_error(struct transport *, char *);
-static struct error * remote_full(struct transport *, char *);
-static struct error * remote_bad_address(struct transport *, char *);
+static struct error * remote_storage_full(struct transport *, char *);
+static struct error * remote_mailbox_unavail(struct transport *, char *);
+static struct error * remote_failure(struct transport *, char *);
+static struct error * remote_unknown_user(struct transport *, char *);
+static struct error * remote_user_not_local(struct transport *, char *);
+static struct error * remote_over_quota(struct transport *, char *);
+static struct error * remote_illegal_user(struct transport *, char *);
 static struct error * write_failed(struct transport *);
 static struct error * read_failed(struct transport *); 
 #else /* not __STDC__ */
@@ -131,8 +155,13 @@ static void catch_timeout();
 static struct error * no_remote();
 static struct error * try_again();
 static struct error * fatal_error();
-static struct error * remote_full();
-static struct error * remote_bad_address();
+static struct error * remote_storage_full();
+static struct error * remote_mailbox_unavail();
+static struct error * remote_failure();
+static struct error * remote_unknown_user();
+static struct error * remote_user_not_local();
+static struct error * remote_over_quota();
+static struct error * remote_illegal_user();
 static struct error * write_failed();
 static struct error * read_failed(); 
 #endif /* not __STDC__ */
@@ -202,11 +231,12 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	 *	421 - closing down  (try again later)
 	 *  5xx - fatal error   (try HELO)
 	 */
-	smtp_out.i = 0;
+	STR_CLEAR(&smtp_out);
 	(void) str_printf(&smtp_out, EHLO(primary_name));
 
 	reply = wait_write_command(smtpb, smtpb->short_timeout,
-				   smtp_out.p, smtp_out.i, &reply_text);
+				   STR(&smtp_out), (int) STR_LEN(&smtp_out),
+				   &reply_text);
 
 	if (reply == REPLY_TIMEOUT) {
 	    /* Some gateways just terminate the conection when they
@@ -280,11 +310,15 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		} else if (strncmpic(cp, "XQUE", keywordlength) == 0) {
 		    smtpb->smtp_flags |= ESMTP_queue;
 		} else if (strncmpic(cp, "EXPN", keywordlength) == 0) {
-		    ; /* NOP XXX smail offers this but can ignore it in here */
+		    ; /* NOP XXX smail offers this but we can ignore it in here */
+		} else if (strncmpic(cp, "HELP", keywordlength) == 0) {
+		    ; /* NOP XXX smail offers this but we can ignore it in here */
 		} else {
-		    char sc = *(cp+keywordlength);
-		    DEBUG1(DBG_DRIVER_MID, "destination offers unsupported ESMTP option %s.\n", cp);
-		    *(cp+keywordlength) = sc;
+		    char sc = *(cp + keywordlength);
+
+		    *(cp + keywordlength) = '\0';
+		    DEBUG1(DBG_DRIVER_MID, "destination offers unsupported ESMTP option '%s'.\n", cp);
+		    *(cp + keywordlength) = sc;
 		}
 	      skip_rest_of_line:
 		while (*cp != 0 && *cp != '\n')
@@ -299,12 +333,12 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		   smtpb->smtp_flags & ESMTP_verbose ? " XVRB" : "",
 		   smtpb->smtp_flags & ESMTP_one ? " XONE" : "",
 		   smtpb->smtp_flags & ESMTP_queue ? " XQUE" : "");
-	    if (smtpb->smtp_flags & ESMTP_verbose
-		&& debug >= DBG_DRIVER_MID) {
-		smtp_out.i = 0;
+	    if (smtpb->smtp_flags & ESMTP_verbose && debug >= DBG_DRIVER_MID) {
+		STR_CLEAR(&smtp_out);
 		(void) str_printf(&smtp_out, VERB);
 		reply = wait_write_command(smtpb, smtpb->short_timeout,
-					   smtp_out.p, smtp_out.i, &reply_text);
+					 STR(&smtp_out), (int) STR_LEN(&smtp_out),
+					 &reply_text);
 		if (REPLY_GROUP (reply) == NEGATIVE_TRY_AGAIN) {
 		    /* remote SMTP closed, try again later */
 		    *error_p = try_again(smtpb->tp, reply_text);
@@ -317,7 +351,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	       to handle malformed EHLO replies: */
 # ifndef NO_LOG_EHLO
 	    /* XXX this is really ugly as it leaves the newlines in place... */
-	    write_log(WRITE_LOG_SYS, "destination supports esmtp, but is buggy (%s)",
+	    write_log(WRITE_LOG_SYS, "destination supports esmtp, but is buggy:\n%s",
 		      reply_text);
 # endif /* not NO_LOG_EHLO */
 	    return SMTP_SUCCEED;
@@ -334,11 +368,12 @@ smtp_startup(smtpb, error_p, try_ehlo)
 #ifdef HAVE_EHLO
   try_helo:
 #endif
-    smtp_out.i = 0;
+    STR_CLEAR(&smtp_out);
     (void) str_printf(&smtp_out, HELO(primary_name));
 
     reply = wait_write_command(smtpb, smtpb->short_timeout,
-			       smtp_out.p, (int) smtp_out.i, &reply_text);
+					 STR(&smtp_out), (int) STR_LEN(&smtp_out),
+					 &reply_text);
 
     if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
 	/* remote SMTP closed, try again later */
@@ -347,18 +382,20 @@ smtp_startup(smtpb, error_p, try_ehlo)
     }
     if (reply == REPLY_SEQUENCE_ERROR) {
 	/* ignore 503 Bad sequence of commands after EHLO/RSET/HELO */
-	write_log(WRITE_LOG_SYS, "503 after EHLO/RSET/HELO (%s)",
+	write_log(WRITE_LOG_SYS, "503 after EHLO/RSET/HELO:\n%s",
 		  reply_text);
     } else if (reply != REPLY_OK) {
 #ifdef HAVE_EHLO
 	if (! tried_rset) {
 	    /* The following */
 	    /* fatal error, probably doesn't understand EHLO */
-	    smtp_out.i = 0;
+	    STR_CLEAR(&smtp_out);
 	    (void) str_printf(&smtp_out, "RSET");
 
 	    reply = wait_write_command(smtpb, smtpb->short_timeout,
-				       smtp_out.p, smtp_out.i, &reply_text);
+					 STR(&smtp_out), (int) STR_LEN(&smtp_out),
+					 &reply_text);
+
 	    if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
 		/* remote SMTP closed, try again later */
 		*error_p = try_again(smtpb->tp, reply_text);
@@ -367,7 +404,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		/* Some gateways don't accept any commands before they get
 		 * a HELO, not even RSET.  Fortunately it is usually safe
 		 * to ignore the error messages. */
-		write_log(WRITE_LOG_SYS, "unexpected response to RSET (%s)",
+		write_log(WRITE_LOG_SYS, "unexpected response to RSET:\n%s",
 			  reply_text);
 	    }
 	    /* the RSET command has been accepted; try again with HELO */
@@ -415,7 +452,7 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
     int success;			/* success value from calls */
     struct addr *cur;			/* current address being sent */
     struct addr *next;			/* next addr to send */
-    struct addr *okay;			/* partially successful addrs */
+    struct addr *okay = NULL;		/* partially successful addrs */
     int local_owner = FALSE;            /* TRUE if owner is local */
     struct transport *tp = smtpb->tp;
 
@@ -424,34 +461,28 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	STR_INIT(&smtp_out);
 	smtp_init_flag = TRUE;
     }
-
-    /*
-     * signal that we are sending a new message,
-     * and give the sender address.
-     *
-     * Possible responses:
-     *	250 - okay	    (continue conversation)
-     *	4xx - temporary error (try again later)
-     *  5xx - fatal error   (return message to sender)
-     */
+    STR_CLEAR(&smtp_out);
 
     /*
      * send MAIL command
+     *
+     * signal that we are sending a new message,
+     * and give the sender address.
      */
-    smtp_out.i = 0;
 
     /* The envelope sender is <> for error messages (with <+> used for
-    internally generated ones that are being pushed back into smail).
-    Otherwise, use the message's original sender, unless there is
-    an owner field set for the first address - this field is set for
-    addresses generated by mailing list directors, and should be the
-    same for all addresses in a given call to the transport.
-    
-    If the owner contains no "@", flag it as local so that the
-    visible name gets added below. */
-
+     * internally generated ones that are being pushed back into smail).
+     * Otherwise, use the message's original sender, unless there is an owner
+     * field set for the first address - this field is set for addresses
+     * generated by mailing list directors, and should be the same for all
+     * addresses in a given call to the transport.
+     * 
+     * If the owner contains no "@", flag it as local so that the visible name
+     * gets added below.
+     */
     local_owner = (!error_sender &&
-      addr->owner && strchr(addr->owner, '@') == NULL);
+		   addr->owner &&
+		   !strchr(addr->owner, '@'));
 
     str_printf(&smtp_out, "%s%s%s%s%s",
                MAIL_BEGIN,
@@ -463,10 +494,11 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	       MAIL_END);
 
     /*
-     * send (a guess of) the size of the message to be transported.
-     * Of course the guess could be a bit more educated, but usually
-     * it doesn't matter if it is slightly incorrect.  We simply add
-     * 2% in order to account for \n -> \r\n conversion.
+     * If the remote-SMTP supports the ESMTP size option then send (a guess of)
+     * the size of the message to be transported.  Of course the guess could be
+     * a bit more educated than it currently is, but usually it doesn't matter
+     * if it is slightly incorrect.  We simply add 2% in order to account for
+     * \n -> \r\n conversion.
      */
     if (smtpb->smtp_flags & ESMTP_size) {
 	str_printf(&smtp_out, " SIZE=%lu",
@@ -484,12 +516,15 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
     }
 #endif
 
-    /* give all of the recipient addresses to the remote SMTP */
-    okay = NULL;
-
     reply = write_command_maybe_wait(smtpb, smtpb->long_timeout,
-				     smtp_out.p, (int) smtp_out.i, &reply_text);
-
+				     STR(&smtp_out), (int) STR_LEN(&smtp_out),
+				     &reply_text);
+    /*
+     * Possible responses:
+     *	250 - okay	    (continue conversation)
+     *	4xx - temporary error (try message again later)
+     *  5xx - fatal error   (return message to sender)
+     */
     if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
 	*error_p = try_again(tp, reply_text);
 	insert_addr_list(addr, defer, *error_p);
@@ -503,7 +538,6 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
     }
 
     /* give all of the recipient addresses to the remote SMTP */
-    okay = NULL;
     for (cur = addr; cur; cur = next) {
 	next = cur->succ;
 
@@ -517,43 +551,89 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	 *  550 - unknown user         (return this addr to sender)
 	 *  5xx - failure	       (return message to sender)
 	 */
-	smtp_out.i = 0;
-
+	STR_CLEAR(&smtp_out);
 	str_printf(&smtp_out, "%s%s%s", RCPT_BEGIN, cur->next_addr, RCPT_END);
 
 	reply = write_command_maybe_wait(smtpb, smtpb->long_timeout,
-					 smtp_out.p, (int) smtp_out.i, &reply_text);
+					 STR(&smtp_out), (int) STR_LEN(&smtp_out),
+					 &reply_text);
 
 	if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
-	    if (reply != REPLY_STORAGE_FULL) {
+	    /*
+	     * NOTE: keep these in sync with the pipeline handling below
+	     */
+	    switch (reply) {
+	    case REPLY_MAILBOX_UNAVAIL:
+		cur->error = remote_mailbox_unavail(tp, reply_text);
+		cur->succ = *defer;	/* defer only the current recipient */
+		*defer = cur;
+		break;
+	    case REPLY_REMOTE_FAILURE:
+		cur->error = remote_failure(tp, reply_text);
+		cur->succ = *defer;	/* defer only the current recipient */
+		*defer = cur;
+		break;
+	    case REPLY_STORAGE_FULL:
+		cur->error = remote_storage_full(tp, reply_text);
+		cur->succ = *defer;	/* defer only the current recipient */
+		*defer = cur;
+		break;
+	    default:
 		*error_p = try_again(tp, reply_text);
+		/*
+		 * unknown codes defer all recipients
+		 * XXX is this sane?
+		 */
 		insert_addr_list(cur, defer, *error_p);
 		insert_addr_list(okay, defer, *error_p);
 		do_smtp_shutdown(smtpb, reply);
 		return FAIL;
 	    }
-	    *error_p = remote_full(tp, reply_text);
-	    cur->succ = *defer;
-	    *defer = cur;
 	} else if (REPLY_GROUP(reply) == NEGATIVE_FAILED) {
-	    if (reply != REPLY_UNKNOWN_USER) {
-		*error_p = remote_bad_address(tp, reply_text);
+	    /*
+	     * NOTE: keep these in sync with the pipeline handling below
+	     */
+	    switch (reply) {
+	    case REPLY_UNKNOWN_USER:
+		cur->error = remote_unknown_user(tp, reply_text);
+		cur->succ = *fail;	/* fail only the current recipient */
+		*fail = cur;
+		break;
+	    case REPLY_USER_NOT_LOCAL:
+		cur->error = remote_user_not_local(tp, reply_text);
+		cur->succ = *fail;	/* fail only the current recipient */
+		*fail = cur;
+		break;
+	    case REPLY_OVER_QUOTA:
+		cur->error = remote_over_quota(tp, reply_text);
+		cur->succ = *fail;	/* fail only the current recipient */
+		*fail = cur;
+		break;
+	    case REPLY_ILLEGAL_USER:
+		cur->error = remote_illegal_user(tp, reply_text);
+		cur->succ = *fail;	/* fail only the current recipient */
+		*fail = cur;
+		break;
+	    case REPLY_SEQUENCE_ERROR:
+	    default:
+		*error_p = fatal_error(tp, reply_text);
+		/*
+		 * unknown codes are fatal for all recipients; i.e. don't send
+		 * to anyone at a mis-behaving remote-SMTP site.
+		 */
 		insert_addr_list(cur, fail, *error_p);
 		insert_addr_list(okay, fail, *error_p);
 		do_smtp_shutdown(smtpb, reply);
-		return FAIL;		/* unknown codes are fatal for all recipients */
+		return FAIL;
 	    }
-	    cur->error = remote_bad_address(tp, reply_text);
-	    cur->succ = *fail;
-	    *fail = cur;
 	} else {
-	    /* successful thus far */
+	    /* successful thus far, or pipelining.... */
 	    cur->succ = okay;
 	    okay = cur;
 	}
     }
     if (! okay) {
-	do_smtp_shutdown(smtpb, REPLY_NOT_ACCEPTABLE);
+	do_smtp_shutdown(smtpb, REPLY_TRANS_FAILURE);
 	return FAIL;
     }
 
@@ -566,24 +646,24 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
      *  5xx - fatal error        (return message to sender)
      */
     reply = write_command_maybe_wait(smtpb, smtpb->long_timeout,
-			       DATA, sizeof(DATA) - 1, &reply_text);
+			       DATA_BEGIN, sizeof(DATA_BEGIN) - 1, &reply_text);
 
+    /*
+     * we only "pipeline" the commands up to the DATA_BEGIN....
+     */
     if (smtpb->smtp_flags & ESMTP_pipelining) {
-
-	if (reply != REPLY_OK) {	/* XXX is this not redundant? */
-	    *error_p = try_again(tp, reply_text);
-	    insert_addr_list(addr, defer, *error_p);
-	    do_smtp_shutdown(smtpb, reply);
-	    return FAIL;
-	}
+	addr = okay;			/* get back to where we started */
+	okay = NULL;
 
 	if (flush_command_stream(smtpb, &reply_text) != REPLY_OK) {
 	    *error_p = try_again(tp, reply_text);
 	    insert_addr_list(addr, defer, *error_p);
+	    /* XXX no need to shut down? -- this indicates a write failed */
 	    return FAIL;
 	}
 
 	/* Now read all the responses.  First the MAIL FROM: reply */
+	DEBUG(DBG_DRIVER_MID, "SMTP-pipeline: expecting reply from 'MAIL FROM:'\n");
 	reply = wait_read_response(smtpb, smtpb->long_timeout,
 				   &reply_text);
 	if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
@@ -598,37 +678,60 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	    return FAIL;
 	}
 
-	addr = okay;
-	okay = NULL;
-
 	/* ... next the reply from each RCPT TO: command */
 	for (cur = addr; cur; cur = next) {
 	    next = cur->succ;
 
+	    DEBUG1(DBG_DRIVER_MID, "SMTP-pipeline: expecting reply from 'RCPT TO:%s'\n", cur->next_addr);
 	    reply = wait_read_response(smtpb, smtpb->long_timeout,
 				       &reply_text);
 
+	    /*
+	     * here we should only defer/fail individual recipients else we'll
+	     * leave un-read replies on the input channel.
+	     */
 	    if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
-		if (reply != REPLY_STORAGE_FULL) {
-		    *error_p = try_again(tp, reply_text);
-		    insert_addr_list(cur, defer, *error_p);
-		    insert_addr_list(okay, defer, *error_p);
-		    do_smtp_shutdown(smtpb, reply);
-		    return FAIL;
+		/*
+		 * NOTE: keep these in sync with the non-pipeline handling above
+		 */
+		switch (reply) {
+		case REPLY_MAILBOX_UNAVAIL:
+		    cur->error = remote_mailbox_unavail(tp, reply_text);
+		    break;
+		case REPLY_REMOTE_FAILURE:
+		    cur->error = remote_failure(tp, reply_text);
+		    break;
+		case REPLY_STORAGE_FULL:
+		    cur->error = remote_storage_full(tp, reply_text);
+		    break;
+		default:
+		    cur->error = try_again(tp, reply_text);
+		    break;
 		}
-		*error_p = remote_full(tp, reply_text);
-		cur->succ = *defer;
+		cur->succ = *defer;	/* defer only the current recipient */
 		*defer = cur;
 	    } else if (REPLY_GROUP(reply) == NEGATIVE_FAILED) {
-		if (reply != REPLY_UNKNOWN_USER) {
-		    *error_p = remote_bad_address(tp, reply_text);
-		    insert_addr_list(cur, fail, *error_p);
-		    insert_addr_list(okay, fail, *error_p);
-		    do_smtp_shutdown(smtpb, reply);
-		    return FAIL;	/* unknown codes are fatal for all recipients */
+		/*
+		 * NOTE: keep these in sync with the non-pipeline handling above
+		 */
+		switch (reply) {
+		case REPLY_UNKNOWN_USER:
+		    cur->error = remote_unknown_user(tp, reply_text);
+		    break;
+		case REPLY_USER_NOT_LOCAL:
+		    cur->error = remote_user_not_local(tp, reply_text);
+		    break;
+		case REPLY_OVER_QUOTA:
+		    cur->error = remote_over_quota(tp, reply_text);
+		    break;
+		case REPLY_ILLEGAL_USER:
+		    cur->error = remote_illegal_user(tp, reply_text);
+		    break;
+		default:
+		    cur->error = fatal_error(tp, reply_text);
+		    break;
 		}
-		cur->error = remote_bad_address(tp, reply_text);
-		cur->succ = *fail;
+		cur->succ = *fail;	/* fail only the current recipient */
 		*fail = cur;
 	    } else {
 		/* successful thus far */
@@ -637,23 +740,34 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	    }
 	}
 	/* ... and finally the reply from the DATA command */
+	DEBUG(DBG_DRIVER_MID, "SMTP-pipeline: expecting reply from 'DATA'\n");
 	reply = wait_read_response(smtpb, smtpb->long_timeout,
 				   &reply_text);
     }
 
     if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
+	/* temporary error, defer message */
 	*error_p = try_again(tp, reply_text);
 	insert_addr_list(okay, defer, *error_p);
 	do_smtp_shutdown(smtpb, reply);
 	return FAIL;
-    }
-
-    if (reply != REPLY_START_DATA && reply != REPLY_OK) {
+    } else if (REPLY_GROUP(reply) == NEGATIVE_FAILED) {
 	/* fatal error, return message to sender */
 	*error_p = fatal_error(tp, reply_text);
 	insert_addr_list(okay, fail, *error_p);
 	do_smtp_shutdown(smtpb, reply);
 	return FAIL;
+    } else if (reply != REPLY_START_DATA) {
+	char *p;
+
+	for (p = reply_text; *p; p++) {
+	    if (*p == '\r') *p = '.';
+	    if (*p == '\n') *p = '.';
+	}
+	*error_p = fatal_error(tp, xprintf("599 remote protocol error: expecting 354 (or failure), got: %s", reply_text));
+	insert_addr_list(okay, fail, *error_p);
+	do_smtp_shutdown(smtpb, REPLY_PROTO_ERROR);
+	return REPLY_PROTO_ERROR;
     }
 
     /*
@@ -677,17 +791,36 @@ smtp_send(smtpb, addr, succeed, defer, fail, error_p)
 	return SUCCEED;
     }
 
-    /* finish by sending the final "." */
+    /*
+     * Finish by sending the final "."
+     * Possible responses:
+     *	250 - message accepted
+     *  451 - local error in processing		(try all recipients again later)
+     *  452 - insufficient system storage	(try all recipients again later)
+     *	4xx - other temporary error		(try all recipients again later)
+     *  552 - exceeded storage allocation	(return message to sender)
+     *  554 - Transaction failed		(return message to sender
+     *  5xx - other fatal error			(return message to sender)
+     */
     reply = wait_write_command(smtpb, smtpb->long_timeout,
 			       DATA_END, sizeof(DATA_END)-1, &reply_text);
 
     if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
+	/* temporary error, defer message */
 	*error_p = try_again(tp, reply_text);
 	insert_addr_list(okay, defer, *error_p);
 	do_smtp_shutdown(smtpb, reply);
 	return FAIL;
     }
-
+    switch (reply) {
+    case REPLY_DOWN:
+    case REPLY_TIMEOUT:
+	/* possible error, defer, but note possible duplicate */
+	*error_p = try_again(tp, "499-read error from remote SMTP server after sending '.'.\n499 NOTE: duplicate delivery may occur!");
+	insert_addr_list(okay, defer, *error_p);
+	do_smtp_shutdown(smtpb, reply);
+	return FAIL;
+    }
     if (reply != REPLY_OK) {
 	/* fatal error, return message to sender */
 	*error_p = fatal_error(tp, reply_text);
@@ -869,9 +1002,10 @@ read_response_internal(smtpb, timeout, reply_text)
     VOLATILE unsigned save_alarm;	/* previous alarm value */
     VOLATILE JUMPSIG save_sig;		/* previous alarm signal catcher */
     register int c;
+    register char *rp;
 
     /* reset the input buffer */
-    smtp_in.i = 0;
+    STR_CLEAR(&smtp_in);
 
     save_alarm = alarm(0);
     JUMP_SETSIG(SIGALRM, catch_timeout, &save_sig);
@@ -892,23 +1026,27 @@ read_response_internal(smtpb, timeout, reply_text)
 
     /* loop until the response is completed */
     do {
-	lstart = smtp_in.i;
+	lstart = STR_LEN(&smtp_in);	/* remember the beginning of this line */
 	while ((c = getc(smtpb->in)) != EOF && c != '\n') {
 	    STR_NEXT(&smtp_in, c);
 	}
-	if (smtp_in.i > 0 && smtp_in.p[smtp_in.i - 1] == '\r') {
-	    --smtp_in.i;
+	rp = STR(&smtp_in);		/* get current pointer */
+	if (STR_LEN(&smtp_in) > 0 && rp[STR_LEN(&smtp_in) - 1] == '\r') {
+	    STR_PREV(&smtp_in);
 	}
 	STR_NEXT(&smtp_in, '\n');
+	rp = STR(&smtp_in);		/* get current pointer */
     } while (c != EOF &&
-	     isdigit(smtp_in.p[lstart]) &&
-	     isdigit(smtp_in.p[lstart + 1]) &&
-	     isdigit(smtp_in.p[lstart + 2]) &&
-	     smtp_in.p[lstart + 3] == '-');
+	     isdigit(rp[lstart]) &&
+	     isdigit(rp[lstart + 1]) &&
+	     isdigit(rp[lstart + 2]) &&
+	     rp[lstart + 3] == '-');
 
-    /* replace last newline with a nul byte */
-    if (smtp_in.i > 0)
-	smtp_in.p[smtp_in.i - 1] = '\0';
+    /* replace last newline with a NUL byte */
+    /* XXX or should we? */
+    if (STR_LEN(&smtp_in) > 0) {
+	rp[STR_LEN(&smtp_in) - 1] = '\0';
+    }
 
     /* restore previous alarm catcher and setting */
     (void) alarm(0);
@@ -917,22 +1055,29 @@ read_response_internal(smtpb, timeout, reply_text)
 	(void) alarm(save_alarm);
     }
 
+    /* 
+     * finally it's safe to see if there was a read error....
+     */
     if (c == EOF) {
 	*reply_text = "499 read error from remote SMTP process";
 	return REPLY_TIMEOUT;
     }
-    if (! isdigit(smtp_in.p[lstart]) ||
-	! isdigit(smtp_in.p[lstart + 1]) ||
-	! isdigit(smtp_in.p[lstart + 2]) ||
-	! isspace(smtp_in.p[lstart+3]))
+    /* .... or other protocol error */
+    rp = STR(&smtp_in);
+    if (! isdigit(rp[lstart]) ||
+	! isdigit(rp[lstart + 1]) ||
+	! isdigit(rp[lstart + 2]) ||
+	! isspace(rp[lstart + 3]))
     {
 	*reply_text = "498 protocol error in reply from remote SMTP process";
 	return REPLY_PROTO_ERROR;
     }
 
-    *reply_text = smtp_in.p;
+    *reply_text = rp;
+
+    /* snarf out the reply code for our return value.... */
+    (void) sscanf(rp, "%3x", &success);
 
-    (void) sscanf(*reply_text, "%3x", &success);
     return success;
 }
 
@@ -940,7 +1085,7 @@ read_response_internal(smtpb, timeout, reply_text)
 /* ARGSUSED */
 static void
 catch_timeout(signo)
-	int	signo;
+	int	signo;			/* UNUSED */
 {
     JUMP_LONGJMP(timeout_buf, 1);
 }
@@ -966,9 +1111,7 @@ no_remote(tp, reply_text)
      * RESOLUTION
      *	    Retries should eventually take care of the problem.
      */
-    error_text =
-	xprintf("transport %s: no connection to remote SMTP server: %s",
-		tp->name, reply_text);
+    error_text = xprintf("%s transport failure: no connection to remote SMTP server:\n%s", tp->name, reply_text);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
     return note_error(ERR_172, error_text);
 }
@@ -990,12 +1133,12 @@ try_again(tp, reply_text)
      *	    from the remote side.
      *
      * ACTIONS
-     *      Defer the input addresses.
+     *      Defer the input address(es).
      *
      * RESOLUTION
      *      Retries should eventually take care of the problem.
      */
-    error_text = xprintf("transport %s: %s", tp->name, reply_text);
+    error_text = xprintf("temporary failure from %s transport:\n%s", tp->name, reply_text);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
     return note_error(ERR_151, error_text);
 }
@@ -1023,20 +1166,71 @@ fatal_error(tp, reply_text)
      *      The resolution depends upon the error message.  See RFC822
      *	    for details.
      */
-    error_text = xprintf("transport %s: %s", tp->name, reply_text);
+    error_text = xprintf("fatal error from %s transport:\n%s", tp->name, reply_text);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
     return note_error(ERR_152 | ERR_NSOWNER, error_text);
 }
 
 static struct error *
-remote_full(tp, reply_text)
+remote_mailbox_unavail(tp, reply_text)
     struct transport *tp;
     char *reply_text;
 {
     char *error_text;
 
     /*
-     * ERR_153 - Remote host's storage is full
+     * ERR_187 - Remote mailbox is busy (SMTP 450)
+     *
+     * DESCRIPTION
+     *      The remote host returned status indicating that it
+     *      cannot accept mail for this mailbox at this time.
+     *
+     * ACTIONS
+     *      Defer this recipient.
+     *
+     * RESOLUTION
+     *      Retries should eventually take care of the problem.
+     */
+    error_text = xprintf("remote mailbox busy, reported by %s transport:\n%s", tp->name, reply_text);
+    DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
+    return note_error(ERR_187, error_text);
+}
+
+static struct error *
+remote_failure(tp, reply_text)
+    struct transport *tp;
+    char *reply_text;
+{
+    char *error_text;
+
+    /*
+     * ERR_188 - Remote failure in processing
+     *
+     * DESCRIPTION
+     *      The remote host returned status indicating that it
+     *      is experiencing temporary problems processing this
+     *      recipient address.
+     *
+     * ACTIONS
+     *      Defer this recipient.
+     *
+     * RESOLUTION
+     *      Retries should eventually take care of the problem.
+     */
+    error_text = xprintf("remote processing error reported by %s transport:\n%s", tp->name, reply_text);
+    DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
+    return note_error(ERR_188, error_text);
+}
+
+static struct error *
+remote_storage_full(tp, reply_text)
+    struct transport *tp;
+    char *reply_text;
+{
+    char *error_text;
+
+    /*
+     * ERR_153 - Remote host's storage is full (SMTP 452)
      *
      * DESCRIPTION
      *      The remote host returned status indicating that he
@@ -1051,20 +1245,20 @@ remote_full(tp, reply_text)
      *      the message will eventually be delivered to all
      *      recipients.
      */
-    error_text = xprintf("transport %s: %s", tp->name, reply_text);
+    error_text = xprintf("remote storage full error reported by %s transport:\n%s", tp->name, reply_text);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
     return note_error(ERR_153, error_text);
 }
 
 static struct error *
-remote_bad_address(tp, reply_text)
+remote_unknown_user(tp, reply_text)
     struct transport *tp;
     char *reply_text;
 {
     char *error_text;
 
     /*
-     * ERR_156 - remote host returned bad address status
+     * ERR_156 - remote host returned bad address status (SMTP 550)
      *
      * DESCRIPTION
      *      Status from the remote host indicates an error in the
@@ -1078,11 +1272,91 @@ remote_bad_address(tp, reply_text)
      *      postmaster at the site that generated the error message
      *      should be consulted.
      */
-    error_text = xprintf("transport %s: %s", tp->name, reply_text);
+    error_text = xprintf("%s transport reports unknown user:\n%s", tp->name, reply_text);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
     return note_error(ERR_156 | ERR_NSOWNER, error_text);
 }
 
+static struct error *
+remote_user_not_local(tp, reply_text)
+    struct transport *tp;
+    char *reply_text;
+{
+    char *error_text;
+
+    /*
+     * ERR_189 - remote host says user not local (SMTP 551)
+     *
+     * DESCRIPTION
+     *      Status from the remote host indicates this user is not local.
+     *
+     * ACTIONS
+     *      Return mail to the address owner or to the sender.
+     *
+     * RESOLUTION
+     *      An alternate address should be attempted or the
+     *      postmaster at the site that generated the error message
+     *      should be consulted.
+     */
+    error_text = xprintf("%s transport reports user not local:\n%s", tp->name, reply_text);
+    DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
+    return note_error(ERR_189 | ERR_NSOWNER, error_text);
+}
+
+static struct error *
+remote_over_quota(tp, reply_text)
+    struct transport *tp;
+    char *reply_text;
+{
+    char *error_text;
+
+    /*
+     * ERR_190 - remote host says user over quota (SMTP 552)
+     *
+     * DESCRIPTION
+     *      Status from the remote host indicates this user has exceeded
+     *      their maximum allowed mailbox size.
+     *
+     * ACTIONS
+     *      Return mail to the address owner or to the sender.
+     *
+     * RESOLUTION
+     *      Out-of-band communications should be attempted or the
+     *      postmaster at the site that generated the error message
+     *      should be consulted.
+     */
+    error_text = xprintf("%s transport reports user over quota:\n%s", tp->name, reply_text);
+    DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
+    return note_error(ERR_190 | ERR_NSOWNER, error_text);
+}
+
+static struct error *
+remote_illegal_user(tp, reply_text)
+    struct transport *tp;
+    char *reply_text;
+{
+    char *error_text;
+
+    /*
+     * ERR_191 - remote host says user is illegal (SMTP 553)
+     *
+     * DESCRIPTION
+     *      Status from the remote host indicates this user account
+     *      is not allowed to receive mail (or the syntax is incorrect).
+     *
+     * ACTIONS
+     *      Return mail to the address owner or to the sender.
+     *
+     * RESOLUTION
+     *      Out-of-band communications should be attempted or the
+     *      postmaster at the site that generated the error message
+     *      should be consulted.
+     */
+    error_text = xprintf("%s transport reports illegal user:\n%s", tp->name, reply_text);
+    DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
+    return note_error(ERR_191 | ERR_NSOWNER, error_text);
+}
+
 static struct error *
 write_failed(tp)
     struct transport *tp;
@@ -1102,8 +1376,7 @@ write_failed(tp)
      * RESOLUTION
      *      Hopefully retries should take care of the problem.
      */
-    error_text = xprintf("transport %s: Error writing to remote host",
-			 tp->name);
+    error_text = xprintf("%s transport failure: Error writing to remote host", tp->name);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
     return note_error(ERR_154, error_text);
 }
@@ -1128,8 +1401,7 @@ read_failed(tp)
      * RESOLUTION
      *      "This should never happen".
      */
-    error_text = xprintf("transport %s: Error writing to remote host",
-			 tp->name);
+    error_text = xprintf("%s transport failure: Error reading from spool file", tp->name);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
     return note_error(ERR_155 | ERR_CONFERR, error_text);
 }
