@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_106:tcpsmtp.c,v 1.37 1998/11/20 00:15:34 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_107:tcpsmtp.c,v 1.38 1999/08/08 16:46:12 woods Exp"
  */
 
 /*
@@ -633,18 +633,18 @@ tcpsmtp_connect(remote_host, ip_addr, family, service, error)
         return -1;
     }
 
-    if (listen_name) {
+    if (sending_name) {
 	struct hostent *hostentp;
 	struct sockaddr_in lsin;	/* local (source) inet socket address */
 
-	if (!(hostentp = gethostbyname(listen_name))) {
+	if (!(hostentp = gethostbyname(sending_name))) {
 	    if (save_error) {
 		xfree(save_error);
 	    }
 #if defined(HAVE_BIND) && !defined(OBSOLETE_RESOLVER)
-	    *error = save_error = xprintf("gethostbyname(%s): %s", listen_name, hstrerror(h_errno));
+	    *error = save_error = xprintf("gethostbyname(%s): %s", sending_name, hstrerror(h_errno));
 #else
-	    *error = save_error = xprintf("gethostbyname(%s): failed.", listen_name);
+	    *error = save_error = xprintf("gethostbyname(%s): failed.", sending_name);
 #endif
 	    return -1;
 	}
@@ -652,12 +652,12 @@ tcpsmtp_connect(remote_host, ip_addr, family, service, error)
 	memcpy((char *) &lsin.sin_addr, hostentp->h_addr_list[0], sizeof(struct in_addr));
 	lsin.sin_family = family;
 	lsin.sin_port = 0;		/* any old port will do.... */
-	DEBUG2(DBG_DRIVER_MID, "\nbinding socket to local source addr (%s)[%s]... ", listen_name, inet_ntoa(lsin.sin_addr));
+	DEBUG2(DBG_DRIVER_MID, "\nbinding socket to local source addr (%s)[%s]... ", sending_name, inet_ntoa(lsin.sin_addr));
 	if (bind(s, (struct sockaddr *) &lsin, sizeof(lsin)) < 0) {
 	    if (save_error) {
 		xfree(save_error);
 	    }
-	    *error = save_error = xprintf("bind(%s[%s]): %s", listen_name, inet_ntoa(lsin.sin_addr), strerror(errno));
+	    *error = save_error = xprintf("bind(%s[%s]): %s", sending_name, inet_ntoa(lsin.sin_addr), strerror(errno));
 	    return -1;
 	}
     }
