@@ -1,18 +1,18 @@
 #! /bin/sh
 :
-#ident	"@(#)smail/util:RELEASE-3_2_0_106:checkerr.sh,v 1.11 1999/02/05 02:04:21 woods Exp"
+#ident	"@(#)smail/util:RELEASE-3_2_0_107:checkerr.sh,v 1.12 1999/08/05 02:35:11 woods Exp"
 #
 #    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
 #    Copyright (C) 1992  Ronald S. Karr
 # See the file COPYING, distributed with smail, for restriction
 # and warranty information.
 
-# Check for errors which have been deposited in the smail error directory.
-# If any new messages are found in this directory, save information
-# related to those errors in the file .checkerror which is then mailed
-# to the postmaster.  If the mail cannot be sent to the postmaster now,
-# such as due to a configuration error, then try to send it in a future
-# invocation of this script.
+# Check for errors which have been deposited in the smail error
+# directory.  If any new messages are found in this directory, save
+# information related to those errors in the file .checkerror which is
+# then mailed to the postmaster.  If the mail cannot be sent to the
+# postmaster now, such as due to a configuration error, then try to
+# send it in a future invocation of this script.
 
 PATH="X_UTIL_PATH_X:X_SECURE_PATH_X"; export PATH
 SMAIL_PROGRAM="X_SMAIL_NAME_X"
@@ -20,10 +20,12 @@ SPOOL_DIRS="`$SMAIL_PROGRAM -bP spool_dirs`"
 TMPDIR="X_TMP_DIR_X"
 HOSTNAME="`$SMAIL_PROGRAM -bP primary_name`"
 LOGFILE="`$SMAIL_PROGRAM -bP logfile`"
+# Old fashioned smaillog form
 OLD_LOGFILE="`echo $LOGFILE | sed -e 's,^\(.*\)/\([^/][^/]*\)$,\1/OLD/\2,'`"
 PANICLOG="`$SMAIL_PROGRAM -bP paniclog`"
+# Old fashioned smaillog form
 OLD_PANICLOG="`echo $PANICLOG | sed -e 's,^\(.*\)/\([^/][^/]*\)$,\1/OLD/\2,'`"
-Z_DOT="X_DOT_Z_X"
+DOT_Z="X_DOT_Z_X"
 ZCAT="X_ZCAT_X"
 PROG=$0
 
@@ -116,27 +118,41 @@ while read SPOOLDIR; do
 	: > .lasttimedone
 	cd error; find * -name '[0-9]*' -print
     fi 2> /dev/null | while read f; do
-	echo ""; echo "------------------ Message $f ------------------"
+	echo ""
+	echo "------------------ Message $f ------------------"
 	if [ -s msglog/$f ]; then
 	    echo "The per-message log file contains:"
 	    sed 's/^/ /' < msglog/$f
 	else
 	    echo "No per-message log file was created"
 	fi
-	echo ""; echo "Logfile entries related to this message are:"
-	(if [ -f $LOGFILE ]; then
+	echo ""
+	echo "Logfile entries related to this message are:"
+	(
+	    if [ -f $LOGFILE ]; then
 		cat $LOGFILE;
-	 fi
-	 if [ -f $OLD_LOGFILE.0 ]; then
-		cat $OLD_LOGFILE.0;
-	 elif [ -f $OLD_LOGFILE.0$Z_DOT ]; then
-		$ZCAT $OLD_LOGFILE.0$Z_DOT;
-	 fi
-	 if [ -f $OLD_LOGFILE.1 ]; then
+	    fi
+	    if [ -f $LOGFILE.0 ]; then
+		cat $LOGFILE.0;			# aka newsyslog
+	    elif [ -f $LOGFILE.0$DOT_Z ]; then
+		$ZCAT $LOGFILE.0$DOT_Z;		# aka newsyslog
+	    fi
+	    if [ -f $OLD_LOGFILE.0 ]; then
+		cat $OLD_LOGFILE.0;		# aka smaillog
+	    elif [ -f $OLD_LOGFILE.0$DOT_Z ]; then
+		$ZCAT $OLD_LOGFILE.0$DOT_Z;	# aka smaillog
+	    fi
+	    if [ -f $LOGFILE.1 ]; then
+		cat $LOGFILE.1;
+	    elif [ -f $LOGFILE.1$DOT_Z ]; then
+		$ZCAT $LOGFILE.1$DOT_Z;
+	    fi
+	    if [ -f $OLD_LOGFILE.1 ]; then
 		cat $OLD_LOGFILE.1;
-	 elif [ -f $OLD_LOGFILE.1$Z_DOT ]; then
-		$ZCAT $OLD_LOGFILE.1$Z_DOT;
-	 fi) | sed ":l1
+	    elif [ -f $OLD_LOGFILE.1$DOT_Z ]; then
+		$ZCAT $OLD_LOGFILE.1$DOT_Z;
+	    fi
+	) | sed ":l1
 		    /^[^|].*\[m$f\]/{
 			:l2
 			s/^/  /
@@ -146,20 +162,33 @@ while read SPOOLDIR; do
 			b l2
 		    }
 		    d"
-	echo ""; echo "Paniclog entries related to this messsage are:"
-	(if [ -f $PANICLOG ]; then
+	echo ""
+	echo "Paniclog entries related to this messsage are:"
+	(
+	    if [ -f $PANICLOG ]; then
 		cat $PANICLOG;
-	 fi
-	 if [ -f $OLD_PANICLOG.0 ]; then
-		cat $OLD_PANICLOG.0;
-	 elif [ -f $OLD_PANICLOG.0$Z_DOT ]; then
-		$ZCAT $OLD_PANICLOG.0$Z_DOT;
-	 fi
-	 if [ -f $OLD_PANICLOG.1 ]; then
+	    fi
+	    if [ -f $PANICLOG.0 ]; then
+		cat $PANICLOG.0;		# aka newsyslog
+	    elif [ -f $PANICLOG.0$DOT_Z ]; then
+		$ZCAT $PANICLOG.0$DOT_Z;	# aka newsyslog
+	    fi
+	    if [ -f $OLD_PANICLOG.0 ]; then
+		cat $OLD_PANICLOG.0;		# aka smaillog
+	    elif [ -f $OLD_PANICLOG.0$DOT_Z ]; then
+		$ZCAT $OLD_PANICLOG.0$DOT_Z;	# aka smaillog
+	    fi
+	    if [ -f $PANICLOG.1 ]; then
+		cat $PANICLOG.1;
+	    elif [ -f $PANICLOG.1$DOT_Z ]; then
+		$ZCAT $PANICLOG.1$DOT_Z;
+	    fi
+	    if [ -f $OLD_PANICLOG.1 ]; then
 		cat $OLD_PANICLOG.1;
-	 elif [ -f $OLD_PANICLOG.1$Z_DOT ]; then
-		$ZCAT $OLD_PANICLOG.1$Z_DOT;
-	 fi) |	grep "\[m$f\]" | sed 's/^/  /'
+	    elif [ -f $OLD_PANICLOG.1$DOT_Z ]; then
+		$ZCAT $OLD_PANICLOG.1$DOT_Z;
+	    fi
+	) | grep "\[m$f\]" | sed 's/^/  /'
     done > .newerrors
 
     mv .newerrors .checkerror
