@@ -1,4 +1,4 @@
-/* @(#) pathmerge.c,v 1.4 1992/09/06 01:10:35 tron Exp */
+/* @(#) pathmerge.c,v 1.5 1997/12/16 22:07:40 woods Exp */
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
  *    Copyright (C) 1992  Ronald S. Karr
@@ -21,9 +21,12 @@
 #include <stdio.h>
 #include <ctype.h>
 #include "defs.h"
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
+#endif
 
 extern int errno;		/* last system error */
-char *malloc();			/* our storage allocator */
+
 char *program;			/* who we are */
 
 #define MAXLINE	(4096+1+1)	/* max line length + newline + NULL */
@@ -42,6 +45,18 @@ enum state {
 	ready			/* the file has a line ready */
 };
 
+/* functions imported by this file */
+#if !defined(HAVE_STDLIB_H) && !defined(HAVE_MALLOC_H)
+extern char *malloc();
+extern char *realloc();
+extern void free();				/* int on SunOS */
+#endif
+
+/* functions local to this file */
+static int readline();
+static int cmp();
+
+int
 main(argc,argv)
 	register int argc;
 	char *argv[];
@@ -185,6 +200,8 @@ main(argc,argv)
 	 * all done
 	 */
 	exit(0);
+
+	/* NOTREACHED */
 }
 
 /*
@@ -192,7 +209,7 @@ main(argc,argv)
  *
  * returns the new EOFs count
  */
-int
+static int
 readline(name, fd, line, status, eofs, which)
 	char *name[];		/* the names of the files */
 	FILE **fd;		/* per-file file descriptors */
@@ -247,7 +264,7 @@ readline(name, fd, line, status, eofs, which)
  * or greater than the second, returning -1, 0 and 1
  * respectively.
  */
-int
+static int
 cmp(s, t)
 	register char *s;	/* first line */
 	register char *t;	/* second line */
